@@ -9,16 +9,17 @@
 #include "../main/SQLHelper.h"
 #include "../httpclient/HTTPClient.h"
 
-#define TOT_TYPE 6
+#define TOT_TYPE 7
 
 const _STR_DEVICE DevicesType[TOT_TYPE] =
 { 
-	{ 0, "switchBox", "Switch Box",int(pTypeLighting2), int(sTypeAC), int(STYPE_OnOff), "relay" },
-	{ 1, "shutterBox", "Shutter Box", int(pTypeLighting2), int(sTypeAC), int(STYPE_BlindsPercentageInverted), "shutter" },
-	{ 2, "wLightBoxS", "Light Box S", int(pTypeLighting2), int(sTypeAC), int(STYPE_Dimmer), "light" },
-	{ 3, "wLightBox", "Light Box", int(pTypeLimitlessLights), int(sTypeLimitlessRGBW), int(STYPE_Dimmer), "rgbw" },
-	{ 4, "gateBox", "Gate Box", int(pTypeLighting2), int(sTypeAC), int(STYPE_Dimmer), "gate" },
-	{ 5, "dimmerBox", "Dimmer Box", int(pTypeLighting2), int(sTypeAC), int(STYPE_Dimmer), "dimmer" }
+	{ 0, "switchBox", "Switch Box",pTypeLighting2, sTypeAC, STYPE_OnOff, "relay" },
+	{ 1, "shutterBox", "Shutter Box", pTypeLighting2, sTypeAC, STYPE_BlindsPercentageInverted, "shutter" },
+	{ 2, "wLightBoxS", "Light Box S", pTypeLighting2, sTypeAC, STYPE_Dimmer, "light" },
+	{ 3, "wLightBox", "Light Box", pTypeLimitlessLights, sTypeLimitlessRGBW, STYPE_Dimmer, "rgbw" },
+	{ 4, "gateBox", "Gate Box", pTypeGeneral, sTypePercentage, 0, "gate" },
+	{ 5, "dimmerBox", "Dimmer Box", pTypeLighting2, sTypeAC, STYPE_Dimmer, "dimmer" },
+	{ 6, "switchBoxD", "Switch Box D", pTypeLighting2, sTypeAC, STYPE_OnOff, "relay" }
 };
 
 int BleBox::GetDeviceTypeByApiName(const std::string &apiName)
@@ -30,7 +31,7 @@ int BleBox::GetDeviceTypeByApiName(const std::string &apiName)
 			return DevicesType[i].unit;
 		}
 	}
-	_log.Log(LOG_ERROR, "BleBox: unknown device api name({0})", apiName.c_str());
+	_log.Log(LOG_ERROR, "BleBox: unknown device api name(%s)", apiName.c_str());
 	return -1;
 }
 
@@ -105,13 +106,12 @@ void BleBox::GetDevicesState()
 		sstr << "/api/" << DevicesType[itt->second].api_state << "/state";
 		std::string command = sstr.str();
 
-		Json::Value root = SendCommand(itt->first, command);
-		if (root == "")
+		Json::Value root = SendCommand(itt->first, command, 2);
+		if (root.empty())
 			continue;
-
-
-		int node = IPToUInt(itt->first);
-		if (node != 0)
+		
+		int IP = IPToUInt(itt->first);
+		if (IP != 0)
 		{
 			switch (itt->second)
 			{
@@ -122,7 +122,7 @@ void BleBox::GetDevicesState()
 					
 					const bool state = root["state"].asBool();
 
-					SendSwitch(node, itt->second, 255, state, 0, DevicesType[itt->second].name);
+					SendSwitch(IP, 0, 255, state, 0, DevicesType[itt->second].name);
 					break;
 				}
 				case 1:
@@ -132,15 +132,17 @@ void BleBox::GetDevicesState()
 
 					const int state = root["state"].asInt();
 
-					const int currentPos = root["currentPos"].asInt();
-					//	const int desiredPos = root["desiredPos"].asInt();
+					if (IsNodesExist(root, "currentPos", "position") == false)
+						break;
+
+					const int currentPos = root["currentPos"]["position"].asInt();
 					const int pos = currentPos;
 
 					bool opened = true;
 					if ((state == 2 && pos == 100) || (state == 3))
 						opened = false;
 
-					SendSwitch(node, itt->second, 255, opened, pos, DevicesType[itt->second].name);
+					SendSwitch(IP, 0, 255, opened, pos, DevicesType[itt->second].name);
 					break;
 				}
 				case 2:
@@ -153,7 +155,7 @@ void BleBox::GetDevicesState()
 					sscanf(currentColor.c_str(), "%x", &hexNumber);
 					int level = (int)(hexNumber / (255.0 / 100.0));
 
-					SendSwitch(node, itt->second, 255, level > 0, level, DevicesType[itt->second].name);
+					SendSwitch(IP, 0, 255, level > 0, level, DevicesType[itt->second].name);
 					break;
 				}
 				case 3:
@@ -165,7 +167,7 @@ void BleBox::GetDevicesState()
 					int hexNumber;
 					sscanf(currentColor.c_str(), "%x", &hexNumber);
 
-					SendRGBWSwitch(node, itt->second, 255, hexNumber, true, DevicesType[itt->second].name);
+					SendRGBWSwitch(IP, 0, 255, hexNumber, true, DevicesType[itt->second].name);
 					break;
 				}
 				case 4:
@@ -173,10 +175,9 @@ void BleBox::GetDevicesState()
 					if (IsNodeExists(root, "currentPos") == false)
 						break;
 
-					const int currentPos = root["currentPos"].asInt();
-					int level = (int)(currentPos / (255.0 / 100.0));
+					const float level = root["currentPos"].asFloat();
 
-					SendSwitch(node, itt->second, 255, level > 0, level, DevicesType[itt->second].name);
+					SendPercentageSensor(IP, 1, 255, level, DevicesType[itt->second].name);
 					break;
 				}
 				case 5:
@@ -187,7 +188,27 @@ void BleBox::GetDevicesState()
 					const int currentPos = root["dimmer"]["currentBrightness"].asInt();
 					int level = (int)(currentPos / (255.0 / 100.0));
 
-					SendSwitch(node, itt->second, 255, level > 0, level, DevicesType[itt->second].name);
+					SendSwitch(IP, 0, 255, level > 0, level, DevicesType[itt->second].name);
+					break;
+				}
+				case 6:
+				{
+					if ((IsNodeExists(root, "relays") == false) || (!root["relays"].isArray()))
+						break;
+
+					Json::Value relays = root["relays"];
+					Json::ArrayIndex count = relays.size();
+					for (Json::ArrayIndex index = 0; index < count; index++)
+					{
+						Json::Value relay = relays[index];
+						if ((IsNodeExists(relay, "relay") == false) || (IsNodeExists(relay, "state") == false))
+							break;
+						int relayNumber = relay["relay"].asInt(); // 0 or 1
+						bool currentState = relay["state"].asBool(); // 0 or 1
+						//std::string name = DevicesType[itt->second].name + " " + relay["state"].asString();
+						SendSwitch(IP, relayNumber, 255, currentState, 0, DevicesType[itt->second].name);
+					}
+					
 					break;
 				}
 			}
@@ -251,106 +272,248 @@ bool BleBox::WriteToHardware(const char *pdata, const unsigned char length)
 	{
 		std::string IPAddress = GetDeviceIP(output);
 
-		switch (output->LIGHTING2.unitcode)
+		int type = GetDeviceType(IPAddress);
+
+		if (type != -1)
 		{
-			case 0:
+			switch (type)
 			{
-				std::string state;
-				if (output->LIGHTING2.cmnd == light2_sOn)
+				case 0:
 				{
-					state = "1";
-				}
-				else
-				{
-					state = "0";
-				}
-
-				Json::Value root = SendCommand(IPAddress, "/s/" + state);
-				if (root == "")
-					return false;
-
-				if (IsNodeExists(root, "state") == false)
-					return false;
-
-				if (root["state"].asString() != state)
-				{
-					_log.Log(LOG_ERROR, "BleBox: state not changed!");
-					return false;
-				}
-				break;
-			}
-
-			case 1:
-			{
-				std::string state;
-				if (output->LIGHTING2.cmnd == light2_sOn)
-				{
-					state = "u";
-				}
-				else
-					if (output->LIGHTING2.cmnd == light2_sOff)
+					std::string state;
+					if (output->LIGHTING2.cmnd == light2_sOn)
 					{
-						state = "d";
+						state = "1";
 					}
 					else
 					{
-						int percentage = output->LIGHTING2.level * 100 / 15;
-						state = boost::to_string(percentage);
+						state = "0";
 					}
 
-				Json::Value root = SendCommand(IPAddress, "/s/" + state);
-				if (root == "")
-					return false;
+					Json::Value root = SendCommand(IPAddress, "/s/" + state);
+					if (root.empty())
+						return false;
 
-				if (IsNodeExists(root, "state") == false)
-					return false;
+					if (IsNodeExists(root, "state") == false)
+						return false;
 
-				//if (root["state"].asString() != state)
-				//{
-				//	_log.Log(LOG_ERROR, "BleBox: state not changed!");
-				//	return false;
-				//}
-				break;
-			}
-
-			case 2:
-			{
-				std::string level;
-				if (output->LIGHTING2.cmnd == light2_sOn)
-				{
-					level = "ff";
-				}
-				else
-					if (output->LIGHTING2.cmnd == light2_sOff)
+					if (root["state"].asString() != state)
 					{
-						level = "00";
+						_log.Log(LOG_ERROR, "BleBox: state not changed!");
+						return false;
+					}
+					break;
+				}
+
+				case 1: // shutterbox
+				{
+					int percentage = 0;
+					switch (output->LIGHTING2.cmnd) {
+						case light2_sOn:
+							percentage = 0;
+							break;
+						case light2_sOff:
+							percentage = 100;
+							break;
+						default:
+							percentage = output->LIGHTING2.level * 100 / 15;
+							break;
+					}
+					
+
+					Json::Value root = SendCommand(IPAddress, "/s/p/" + boost::to_string(percentage));
+ 
+ 					if (root.empty())
+  						return false;
+					
+					if (IsNodeExists(root, "state") == false)
+						return false;
+
+					// TODO - add check
+					//if (root["state"].asString() != state)
+					//{
+					//	_log.Log(LOG_ERROR, "BleBox: state not changed!");
+					//	return false;
+					//}
+					break;
+				}
+
+				case 2:
+				{
+					std::string level;
+					if (output->LIGHTING2.cmnd == light2_sOn)
+					{
+						level = "ff";
+					}
+					else
+						if (output->LIGHTING2.cmnd == light2_sOff)
+						{
+							level = "00";
+						}
+						else
+						{
+							int percentage = output->LIGHTING2.level * 255 / 15;
+
+							char value[4];
+							sprintf(value, "%x", percentage);
+							level = value;
+						}
+
+					Json::Value root = SendCommand(IPAddress, "/s/" + level);
+					if (root.empty())
+						return false;
+
+					if (IsNodesExist(root, "light", "desiredColor") == false)
+						return false;
+
+					if (root["light"]["desiredColor"].asString() != level)
+					{
+						_log.Log(LOG_ERROR, "BleBox: light not changed!");
+						return false;
+					}
+
+					break;
+				}
+
+				case 5: // dimmerBox
+				{
+					std::string level;
+					if (output->LIGHTING2.cmnd == light2_sOn)
+					{
+						level = "ff";
+					}
+					else
+						if (output->LIGHTING2.cmnd == light2_sOff)
+						{
+							level = "0";
+						}
+						else
+						{
+							int percentage = output->LIGHTING2.level * 255 / 15;
+
+							char value[4];
+							sprintf(value, "%x", percentage);
+							level = value;
+						}
+
+					Json::Value root = SendCommand(IPAddress, "/s/" + level);
+					if (root.empty())
+						return false;
+
+					if (IsNodesExist(root, "dimmer", "desiredBrightness") == false)
+						return false;
+
+					std::stringstream ss;
+					ss << std::hex << root["dimmer"]["desiredBrightness"].asInt();
+					std::string state = ss.str();
+
+					if (state != level)
+					{
+						_log.Log(LOG_ERROR, "BleBox: dimmer not changed!");
+						return false;
+					}
+
+					break;
+				}
+
+				case 6: //switchboxd
+				{
+					std::string state;
+					std::string relayNumber;
+					if (output->LIGHTING2.cmnd == light2_sOn)
+					{
+						state = "1";
 					}
 					else
 					{
-						int percentage = output->LIGHTING2.level * 255 / 15;
-
-						char value[4];
-						sprintf(value, "%x", percentage);
-						level = value;
+						state = "0";
+					}
+					if (output->LIGHTING2.unitcode == 0)
+					{
+						relayNumber = "0";
+					}
+					else
+					{
+						relayNumber = "1";
 					}
 
-				Json::Value root = SendCommand(IPAddress, "/s/" + level);
-				if (root == "")
-					return false;
+					std::stringstream ss;
+					ss << "/s/" << relayNumber << "/" << state;
 
-				if (IsNodesExist(root, "light", "currentColor") == false)
-					return false;
+					Json::Value root = SendCommand(IPAddress, ss.str());
+					if (root.empty())
+						return false;
 
-				if (root["light"]["currentColor"].asString() != level) // TODO or desiredcolor ??
+					if ((IsNodeExists(root, "relays") == false) || (!root["relays"].isArray()))
+						return false;
+
+					bool success = false;
+					Json::Value relays = root["relays"];
+					Json::ArrayIndex count = relays.size();
+					for (Json::ArrayIndex index = 0; index < count; index++)
+					{
+						Json::Value relay = relays[index];
+						if ((IsNodeExists(relay, "relay") == false) || (IsNodeExists(relay, "state") == false))
+							continue;
+						int relayNumber = relay["relay"].asInt(); // 0 or 1
+						std::string currentState = relay["state"].asString(); // 0 or 1
+						if (((output->LIGHTING2.unitcode) == relayNumber) && (state == currentState))
+						{
+							success = true;
+							break;
+						}
+					}
+					return success;
+
+					break;
+				}
+				default:
 				{
-					_log.Log(LOG_ERROR, "BleBox: light not changed!");
 					return false;
 				}
-
-				break;
 			}
+			return true;
 		}
 	}
+
+	if (output->ICMND.packettype == pTypeGeneralSwitch && output->LIGHTING2.subtype == sTypeAC)
+	{
+		std::string IPAddress = GetDeviceRevertIP(output);
+
+		int type = GetDeviceType(IPAddress);
+
+		switch (type)
+		{
+			case 4: // gatebox
+			{
+				std::string command;
+				if (output->ICMND.msg4 == 2) // 2=primary, 3=secondary
+				{
+					command = "p";
+				}
+				else
+				{
+					command = "s";
+				}
+
+				Json::Value root = SendCommand(IPAddress, "/s/" + command);
+				if (root.empty())
+					return false;
+
+				if (IsNodeExists(root, "gate") == false)
+					return false;
+
+				break;
+			}
+			default:
+			{
+				return false;
+			}
+
+		}
+		return true;
+	}
+
 	if (output->ICMND.packettype == pTypeLimitlessLights && output->LIGHTING2.subtype == sTypeLimitlessRGBW)
 	{
 		std::string IPAddress = GetDeviceRevertIP(output);
@@ -365,9 +528,9 @@ bool BleBox::WriteToHardware(const char *pdata, const unsigned char length)
 		std::string state(level);
 
 		Json::Value root = SendCommand(IPAddress, "/s/" + state);
-		if (root == "")
+		if (root.empty())
 			return false;
-	
+
 		if (IsNodesExist(root, "rgbw", "desiredColor") == false)
 			return false;
 
@@ -376,9 +539,10 @@ bool BleBox::WriteToHardware(const char *pdata, const unsigned char length)
 			_log.Log(LOG_ERROR, "BleBox: rgbw not changed!");
 			return false;
 		}
+		return true;
 	}
-	
-	return true;
+
+	return false;
 }
 
 bool BleBox::IsNodeExists(const Json::Value root, const std::string node)
@@ -440,8 +604,6 @@ void BleBox::SendSwitch(const int NodeID, const int ChildID, const int BatteryLe
 		int nvalue = atoi(result[0][1].c_str());
 		if ((!bOn) && (nvalue == light2_sOff))
 			return;
-		if (bOn && (nvalue == light2_sOn))
-			return;
 		if ((bOn && (nvalue != light2_sOff)))
 		{
 			//Check Level
@@ -487,7 +649,8 @@ namespace http {
 		{
 			if (session.rights != 2)
 			{
-				return;
+				session.reply_status = reply::forbidden;
+				return; //Only admin user allowed
 			}
 
 			std::string hwid = request::findValue(&req, "idx");
@@ -499,7 +662,7 @@ namespace http {
 			root["title"] = "BleBoxGetNodes";
 
 			std::vector<std::vector<std::string> > result;
-			result = m_sql.safe_query("SELECT ID,Name,DeviceID,Unit FROM DeviceStatus WHERE (HardwareID=='%d')", pBaseHardware->m_HwdID);
+			result = m_sql.safe_query("SELECT ID,Name,DeviceID FROM DeviceStatus WHERE (HardwareID=='%d')", pBaseHardware->m_HwdID);
 			if (result.size() > 0)
 			{
 				std::vector<std::vector<std::string> >::const_iterator itt;
@@ -516,7 +679,32 @@ namespace http {
 					root["result"][ii]["idx"] = sd[0];
 					root["result"][ii]["Name"] = sd[1];
 					root["result"][ii]["IP"] = ip;
-					root["result"][ii]["Type"] = DevicesType[atoi(sd[3].c_str())].name;
+					root["result"][ii]["Type"] = "unknown";
+					root["result"][ii]["Uptime"] = "unknown";			
+					root["result"][ii]["hv"] = "unknown";
+					root["result"][ii]["fv"] = "unknown";
+
+					BleBox *pHardware = reinterpret_cast<BleBox*>(pBaseHardware);
+
+					int type = pHardware->GetDeviceType(ip);
+					if (type != -1)
+					{
+						root["result"][ii]["Type"] = DevicesType[type].name;
+
+						Json::Value state = pHardware->GetApiDeviceState(ip);
+						if (!state.isNull())
+						{
+							if (pHardware->IsNodeExists(state, "fv"))
+								root["result"][ii]["fv"] = state["fv"].asString();
+
+							if (pHardware->IsNodeExists(state, "hv"))
+								root["result"][ii]["hv"] = state["hv"].asString();
+
+							std::string uptime = pHardware->GetUptime(ip);
+							root["result"][ii]["Uptime"] = uptime;
+						}
+					}
+
 					ii++;
 				}
 			}
@@ -527,7 +715,8 @@ namespace http {
 		{
 			if (session.rights != 2)
 			{
-				return;
+				session.reply_status = reply::forbidden;
+				return; //Only admin user allowed
 			}
 
 			std::string hwid = request::findValue(&req, "idx");
@@ -560,7 +749,8 @@ namespace http {
 		{
 			if (session.rights != 2)
 			{
-				return;
+				session.reply_status = reply::forbidden;
+				return; //Only admin user allowed
 			}
 
 			std::string hwid = request::findValue(&req, "idx");
@@ -586,7 +776,8 @@ namespace http {
 		{
 			if (session.rights != 2)
 			{
-				return;
+				session.reply_status = reply::forbidden;
+				return; //Only admin user allowed
 			}
 
 			std::string hwid = request::findValue(&req, "idx");
@@ -615,7 +806,8 @@ namespace http {
 		{
 			if (session.rights != 2)
 			{
-				return;
+				session.reply_status = reply::forbidden;
+				return; //Only admin user allowed
 			}
 
 			std::string hwid = request::findValue(&req, "idx");
@@ -640,7 +832,8 @@ namespace http {
 		{
 			if (session.rights != 2)
 			{
-				return;
+				session.reply_status = reply::forbidden;
+				return; //Only admin user allowed
 			}
 
 			std::string hwid = request::findValue(&req, "idx");
@@ -654,36 +847,81 @@ namespace http {
 			pHardware->RemoveAllNodes();
 		}
 
+		void CWebServer::Cmd_BleBoxAutoSearchingNodes(WebEmSession & session, const request& req, Json::Value &root)
+		{
+			if (session.rights != 2)
+			{
+				session.reply_status = reply::forbidden;
+				return; //Only admin user allowed
+			}
+
+			std::string hwid = request::findValue(&req, "idx");
+			std::string ipmask = request::findValue(&req, "ipmask");
+			if (
+				(hwid == "") ||
+				(ipmask == "")
+				)
+				return;
+			CDomoticzHardwareBase *pBaseHardware = m_mainworker.GetHardwareByIDType(hwid, HTYPE_BleBox);
+			if (pBaseHardware == NULL)
+				return;
+			BleBox *pHardware = reinterpret_cast<BleBox*>(pBaseHardware);
+
+			root["status"] = "OK";
+			root["title"] = "BleBoxAutoSearchingNodes";
+			pHardware->SearchNodes(ipmask);
+		}
+
+		void CWebServer::Cmd_BleBoxUpdateFirmware(WebEmSession & session, const request& req, Json::Value &root)
+		{
+			if (session.rights != 2)
+			{
+				session.reply_status = reply::forbidden;
+				return; //Only admin user allowed
+			}
+
+			std::string hwid = request::findValue(&req, "idx");
+			CDomoticzHardwareBase *pBaseHardware = m_mainworker.GetHardwareByIDType(hwid, HTYPE_BleBox);
+			if (pBaseHardware == NULL)
+				return;
+			BleBox *pHardware = reinterpret_cast<BleBox*>(pBaseHardware);
+
+			root["status"] = "OK";
+			root["title"] = "BleBoxUpdateFirmware";
+			pHardware->UpdateFirmware();
+		}
+
 	}
 }
 
-Json::Value BleBox::SendCommand(const std::string &IPAddress, const std::string &command)
+Json::Value BleBox::SendCommand(const std::string &IPAddress, const std::string &command, const int timeOut)
 {
-	std::vector<std::string> extraHeaders;
 	std::string result;
+	Json::Value root;
 
 	std::stringstream sstr;
 	sstr << "http://" << IPAddress << command;
 
 	std::string sURL = sstr.str();
-	if (!HTTPClient::GET(sURL, extraHeaders, result))
+	HTTPClient::SetConnectionTimeout(timeOut);
+	HTTPClient::SetTimeout(4);
+	if (!HTTPClient::GET(sURL, result))
 	{
 		_log.Log(LOG_ERROR, "BleBox: send '%s'command to %s failed!", command.c_str(), IPAddress.c_str());
-		return "";
+		return root;
 	}
 
-	Json::Value root;
 	Json::Reader jReader;
 	if (!jReader.parse(result, root))
 	{
 		_log.Log(LOG_ERROR, "BleBox: Invalid json received!");
-		return "";
+		return root;
 	}
 
 	if (root.size() == 0)
 	{
 		_log.Log(LOG_ERROR, "BleBox: Json is empty!");
-		return "";
+		return root;
 	}
 
 	if (root.isArray())
@@ -694,8 +932,8 @@ Json::Value BleBox::SendCommand(const std::string &IPAddress, const std::string
 
 std::string BleBox::IdentifyDevice(const std::string &IPAddress)
 {
-	Json::Value root = SendCommand(IPAddress, "/api/device/state");
-	if (root == "")
+	Json::Value root = SendCommand(IPAddress, "/api/device/state", 2);
+	if (!root.isObject())
 		return "";
 
 	std::string result;
@@ -719,6 +957,62 @@ std::string BleBox::IdentifyDevice(const std::string &IPAddress)
 	return result;
 }
 
+Json::Value BleBox::GetApiDeviceState(const std::string &IPAddress)
+{
+	Json::Value empty;
+
+	Json::Value root = SendCommand(IPAddress, "/api/device/state", 2);
+	if (!root.isObject())
+		return empty;
+
+	if (root["device"].empty() == true)
+	{
+		return root;
+	}
+	else
+	{
+		return root["device"];
+	}
+}
+
+std::string BleBox::GetUptime(const std::string &IPAddress)
+{
+	Json::Value root = SendCommand(IPAddress, "/api/device/uptime", 2);
+	if (root.empty())
+		return "unknown";
+
+	if (root["uptime"].empty() == true)
+		return "unknown";
+
+	std::string result;
+
+	uint64_t msec = root["uptime"].asUInt64();
+	char timestring[32] = "";
+
+	uint64_t total_minutes = msec / (1000 * 60);
+	int days = static_cast<int>(total_minutes / (24 * 60));
+	int hours = static_cast<int>(total_minutes / 60 - days * 24);
+	int mins = static_cast<int>(total_minutes - days * 24 * 60 - hours * 60);   //sec / 60 - day * (24 * 60) - hour * 60;
+	
+	sprintf(timestring, "%d:%02d:%02d", days, hours, mins);
+
+	return timestring;
+}
+
+int BleBox::GetDeviceType(const std::string &IPAddress)
+{
+	std::map<const std::string, const int>::const_iterator itt = m_devices.find(IPAddress);
+	if (itt == m_devices.end())
+	{
+		_log.Log(LOG_ERROR, "BleBox: unknown device (%s)", IPAddress.c_str());
+		return -1;
+	}
+	else
+	{
+		return itt->second;
+	}
+}
+
 void BleBox::AddNode(const std::string &name, const std::string &IPAddress)
 {
 	std::string deviceApiName = IdentifyDevice(IPAddress);
@@ -733,11 +1027,43 @@ void BleBox::AddNode(const std::string &name, const std::string &IPAddress)
 
 	std::string szIdx = IPToHex(IPAddress, deviceType.deviceID);
 
-	m_sql.safe_query(
-		"INSERT INTO DeviceStatus (HardwareID, DeviceID, Unit, Type, SubType, SwitchType, Used, SignalLevel, BatteryLevel, Name, nValue, sValue) "
-		"VALUES (%d, '%q', %d, %d, %d, %d, 1, 12, 255, '%q', 0, 'Unavailable')",
-		m_HwdID, szIdx.c_str(), deviceType.unit, deviceType.deviceID, deviceType.subType, deviceType.switchType, name.c_str());
+	if (deviceType.unit == 4) // gatebox
+	{
+		m_sql.safe_query(
+			"INSERT INTO DeviceStatus (HardwareID, DeviceID, Unit, Type, SubType, SwitchType, Used, SignalLevel, BatteryLevel, Name, nValue, sValue) "
+			"VALUES (%d, '%q', %d, %d, %d, %d, 1, 12, 255, '%q', 0, 'Unavailable')",
+			m_HwdID, szIdx.c_str(), 1, deviceType.deviceID, deviceType.subType, deviceType.switchType, name.c_str());
 
+		m_sql.safe_query(
+			"INSERT INTO DeviceStatus (HardwareID, DeviceID, Unit, Type, SubType, SwitchType, Used, SignalLevel, BatteryLevel, Name, nValue, sValue) "
+			"VALUES (%d, '%q', %d, %d, %d, %d, 1, 12, 255, '%q', 0, 'Unavailable')",
+			m_HwdID, szIdx.c_str(), 2, pTypeGeneralSwitch, sTypeAC, STYPE_PushOn, name.c_str());
+
+		m_sql.safe_query(
+			"INSERT INTO DeviceStatus (HardwareID, DeviceID, Unit, Type, SubType, SwitchType, Used, SignalLevel, BatteryLevel, Name, nValue, sValue) "
+			"VALUES (%d, '%q', %d, %d, %d, %d, 1, 12, 255, '%q', 0, 'Unavailable')",
+			m_HwdID, szIdx.c_str(), 3, pTypeGeneralSwitch, sTypeAC, STYPE_PushOn, name.c_str());
+	}
+	else
+	if (deviceType.unit == 6) // switchboxd
+	{
+		m_sql.safe_query(
+			"INSERT INTO DeviceStatus (HardwareID, DeviceID, Unit, Type, SubType, SwitchType, Used, SignalLevel, BatteryLevel, Name, nValue, sValue) "
+			"VALUES (%d, '%q', %d, %d, %d, %d, 1, 12, 255, '%q', 0, 'Unavailable')",
+			m_HwdID, szIdx.c_str(), 0, deviceType.deviceID, deviceType.subType, deviceType.switchType, name.c_str());
+
+		m_sql.safe_query(
+			"INSERT INTO DeviceStatus (HardwareID, DeviceID, Unit, Type, SubType, SwitchType, Used, SignalLevel, BatteryLevel, Name, nValue, sValue) "
+			"VALUES (%d, '%q', %d, %d, %d, %d, 1, 12, 255, '%q', 0, 'Unavailable')",
+			m_HwdID, szIdx.c_str(), 1, deviceType.deviceID, deviceType.subType, deviceType.switchType, name.c_str());
+	}
+	else
+	{
+		m_sql.safe_query(
+			"INSERT INTO DeviceStatus (HardwareID, DeviceID, Unit, Type, SubType, SwitchType, Used, SignalLevel, BatteryLevel, Name, nValue, sValue) "
+			"VALUES (%d, '%q', %d, %d, %d, %d, 1, 12, 255, '%q', 0, 'Unavailable')",
+			m_HwdID, szIdx.c_str(), 0, deviceType.deviceID, deviceType.subType, deviceType.switchType, name.c_str());
+	}
 	ReloadNodes();
 }
 
@@ -788,7 +1114,7 @@ bool BleBox::LoadNodes()
 	boost::lock_guard<boost::mutex> l(m_mutex);
 
 	std::vector<std::vector<std::string> > result;
-	result = m_sql.safe_query("SELECT ID,DeviceID, Unit FROM DeviceStatus WHERE (HardwareID==%d)", m_HwdID);
+	result = m_sql.safe_query("SELECT ID,DeviceID FROM DeviceStatus WHERE (HardwareID==%d)", m_HwdID);
 	if (result.size() > 0)
 	{
 		std::vector<std::vector<std::string> >::const_iterator itt;
@@ -796,8 +1122,16 @@ bool BleBox::LoadNodes()
 		{
 			const std::vector<std::string> &sd = *itt;
 			std::string addressIP = GetDeviceIP(sd[1]);
-			int type = atoi(sd[2].c_str());
-			m_devices.insert(std::pair<const std::string, const int>(addressIP, type));
+
+			std::string deviceApiName = IdentifyDevice(addressIP);
+			if (deviceApiName.empty())
+				continue;
+
+			int deviceTypeID = GetDeviceTypeByApiName(deviceApiName);
+			if (deviceTypeID == -1)
+				continue;
+
+			m_devices.insert(std::pair<const std::string, const int>(addressIP, deviceTypeID));
 		}
 		return true;
 	}
@@ -814,3 +1148,44 @@ void BleBox::ReloadNodes()
 	LoadNodes();
 }
 
+void BleBox::UpdateFirmware()
+{
+	std::map<const std::string, const int>::const_iterator itt;
+	for (itt = m_devices.begin(); itt != m_devices.end(); ++itt)
+	{
+		Json::Value root = SendCommand(itt->first, "/api/ota/update", 2);
+	}
+}
+
+void BleBox::SearchNodes(const std::string &ipmask)
+{ 
+	std::vector<std::string> strarray;
+	StringSplit(ipmask, ".", strarray); // ipmask - expected "x.y.z.*"
+	if (strarray.size() != 4)
+		return;
+	if (strarray[3] != "*")
+		return;
+	if (!isInt(strarray[0]) || !isInt(strarray[1]) || !isInt(strarray[2]))
+		return;
+
+
+	std::vector< boost::shared_ptr<boost::thread> > searchingThreads;
+
+	for (unsigned int i = 1; i < 255; ++i)
+	{
+		std::stringstream sstr;
+		sstr << strarray[0] << "." << strarray[1] << "." << strarray[2] << "." << i;
+		std::string IPAddress = sstr.str();
+
+		std::map<const std::string, const int>::const_iterator itt = m_devices.find(IPAddress);
+		if (itt == m_devices.end())
+		{
+			searchingThreads.push_back(boost::shared_ptr<boost::thread>(new boost::thread(boost::bind(&BleBox::AddNode, this, "unknown", IPAddress))));
+		}
+	}
+
+	for (size_t i = 1; i <= searchingThreads.size(); ++i)
+	{
+		searchingThreads[i-1]->join();
+	}
+}
