@@ -31,6 +31,13 @@ public:
 	void RemoveAllNodes();
 	void SetSettings(const int pollIntervalsec);
 	void Restart();
+	void UpdateFirmware();
+	void SearchNodes(const std::string &ipmask);
+	std::string GetUptime(const std::string &IPAddress);
+	int GetDeviceType(const std::string &IPAddress);
+	Json::Value GetApiDeviceState(const std::string &IPAddress);
+	bool IsNodeExists(const Json::Value root, const std::string node);
+	bool IsNodesExist(const Json::Value root, const std::string node, const std::string value);
 
 private:
 	volatile bool m_stoprequested;
@@ -43,16 +50,13 @@ private:
 	bool StopHardware();
 	void Do_Work();
 
-	bool IsNodeExists(const Json::Value root, const std::string node);
-	bool IsNodesExist(const Json::Value root, const std::string node, const std::string value);
-
 	std::string IdentifyDevice(const std::string &IPAddress);
 	int GetDeviceTypeByApiName(const std::string &apiName);
 	std::string GetDeviceIP(const tRBUF *id);
 	std::string GetDeviceRevertIP(const tRBUF *id);
 	std::string GetDeviceIP(const std::string &id);
 	std::string IPToHex(const std::string &IPAddress, const int type);
-	Json::Value SendCommand(const std::string &IPAddress, const std::string &command);
+	Json::Value SendCommand(const std::string &IPAddress, const std::string &command, const int timeOut = 3);
 	void GetDevicesState();
 
 	void SendSwitch(const int NodeID, const int ChildID, const int BatteryLevel, const bool bOn, const double Level, const std::string &defaultname);
