@@ -105,6 +105,11 @@ const uint8_t nmagic_alarm[] = {
 #define NEFITEASY_RRCCONTACT_PREFIX "rrccontact_"
 #define NEFITEASY_RRCGATEWAY_PREFIX "rrcgateway_"
 
+bool CheckId(const char* szUrlPath, const Json::Value& response)
+{
+	return (!response["id"].empty() && response["id"].asString() == szUrlPath);
+}
+
 CNefitEasy::CNefitEasy(const int ID, const std::string &IPAddress, const unsigned short usIPPort):
 m_szIPAddress(IPAddress)
 {
@@ -365,7 +370,7 @@ bool CNefitEasy::GetStatusDetails()
 
 	Json::Reader jReader;
 	bool ret = jReader.parse(sResult, root);
-	if (!ret)
+	if ((!ret) || (!root.isObject()))
 	{
 		if (sResult.find("Error: REQUEST_TIMEOUT") != std::string::npos)
 			_log.Log(LOG_ERROR, "NefitEasy: Request Timeout !");
@@ -373,6 +378,12 @@ bool CNefitEasy::GetStatusDetails()
 			_log.Log(LOG_ERROR, "NefitEasy: Invalid data received (main)!");
 		return false;
 	}
+	if(!CheckId(NEFITEASY_STATUS_URL, root))
+	{
+		_log.Log(LOG_ERROR, "NefitEasy: Invalid response received (main)");
+		return false;
+	}
+	
 	if (root["value"].empty())
 	{
 		_log.Log(LOG_ERROR, "NefitEasy: Invalid data received (main)");
@@ -505,11 +516,16 @@ bool CNefitEasy::GetOutdoorTemp()
 	SaveString2Disk(sResult, "E:\\nefit_outdoor.json");
 #endif
 	ret = jReader.parse(sResult, root);
-	if (!ret)
+	if ((!ret) || (!root.isObject()))
 	{
 		_log.Log(LOG_ERROR, "NefitEasy: Invalid data received! (ODT)");
 		return false;
 	}
+	if(!CheckId(NEFITEASY_OUTDOORTEMP_URL, root))
+	{
+		_log.Log(LOG_ERROR, "NefitEasy: Invalid response received! (ODT)");
+		return false;
+	}
 	if (root["value"].empty() == true)
 	{
 		_log.Log(LOG_ERROR, "NefitEasy: Invalid data received (ODT)");
@@ -554,11 +570,16 @@ bool CNefitEasy::GetFlowTemp()
 	SaveString2Disk(sResult, "E:\\nefit_flow.json");
 #endif
 	ret = jReader.parse(sResult, root);
-	if (!ret)
+	if ((!ret) || (!root.isObject()))
 	{
 		_log.Log(LOG_ERROR, "NefitEasy: Invalid data received! (FT)");
 		return false;
 	}
+	if(!CheckId(NEFITEASY_FLOWTEMP_URL, root))
+	{
+		_log.Log(LOG_ERROR, "NefitEasy: Invalid response received (FT)");
+		return false;
+	}
 	if (root["value"].empty() == true)
 	{
 		_log.Log(LOG_ERROR, "NefitEasy: Invalid data received (FT)");
@@ -603,11 +624,16 @@ bool CNefitEasy::GetPressure()
 	SaveString2Disk(sResult, "E:\\nefit_uipres.json");
 #endif
 	ret = jReader.parse(sResult, root);
-	if (!ret)
+	if ((!ret) || (!root.isObject()))
 	{
 		_log.Log(LOG_ERROR, "NefitEasy: Invalid data received! (Press)");
 		return false;
 	}
+	if(!CheckId(NEFITEASY_PRESSURE_URL, root))
+	{
+		_log.Log(LOG_ERROR, "NefitEasy: Invalid response received! (Press)");
+		return false;
+	}
 	if (root["value"].empty())
 	{
 		_log.Log(LOG_ERROR, "NefitEasy: Invalid data received (Press)");
@@ -650,11 +676,16 @@ bool CNefitEasy::GetDisplayCode()
 	SaveString2Disk(sResult, "E:\\nefit_displaycode.json");
 #endif
 	ret = jReader.parse(sResult, root);
-	if (!ret)
+	if ((!ret) || (!root.isObject()))
 	{
 		_log.Log(LOG_ERROR, "NefitEasy: Invalid data received! (DP)");
 		return false;
 	}
+	if(!CheckId(NEFITEASY_DISPLAYCODE_URL, root))
+	{
+		_log.Log(LOG_ERROR, "NefitEasy: Invalid response received! (DP)");
+		return false;
+	}
 	if (root["value"].empty() == true)
 	{
 		_log.Log(LOG_ERROR, "NefitEasy: Invalid data received (DP)");
@@ -747,11 +778,16 @@ bool CNefitEasy::GetGasUsage()
 	SaveString2Disk(sResult, "E:\\nefit_yearTotal.json");
 #endif
 	ret = jReader.parse(sResult, root);
-	if (!ret)
+	if ((!ret) || (!root.isObject()))
 	{
 		_log.Log(LOG_ERROR, "NefitEasy: Invalid data received! (Gas)");
 		return false;
 	}
+	if(!CheckId(NEFITEASY_GAS_URL, root))
+	{
+		_log.Log(LOG_ERROR, "NefitEasy: Invalid response received! (Gas)");
+		return false;
+	}
 	if (root["value"].empty())
 	{
 		_log.Log(LOG_ERROR, "NefitEasy: Invalid data received (Gas)");
@@ -828,4 +864,4 @@ void CNefitEasy::SetSetpoint(const int idx, const float temp)
 		return;
 	}
 	GetStatusDetails();
-}
+}
\ No newline at end of file
