@@ -11,26 +11,41 @@ public:
 	P1MeterBase(void);
 	~P1MeterBase(void);
 
-	P1Power	m_p1power;
-	P1Gas	m_p1gas;
+	P1Power	m_power;
+	P1Gas	m_gas;
 private:
-	unsigned long m_lastgasusage;
-	unsigned long m_lastelectrausage;
-	time_t m_lastSharedSendElectra;
-	time_t m_lastSharedSendGas;
+	bool m_bDisableCRC;
+	int m_ratelimit;
 
-	unsigned char m_linecount;
-	unsigned char m_exclmarkfound;
-	unsigned char m_CRfound;
-	void Init();
-	bool MatchLine();
-	void ParseData(const unsigned char *pData, int Len, unsigned char disable_crc);
-	unsigned char m_buffer[1028];
+	unsigned char m_p1version;
+
+	unsigned char m_buffer[1400];
 	int m_bufferpos;
+	unsigned char m_exclmarkfound;
+	unsigned char m_linecount;
+	unsigned char m_CRfound;
+
 	unsigned char l_buffer[128];
 	int l_bufferpos;
 	unsigned char l_exclmarkfound;
 
+	unsigned long m_lastgasusage;
+	time_t m_lastSharedSendGas;
+	time_t m_lastUpdateTime;
+
+	float m_voltagel1;
+	float m_voltagel2;
+	float m_voltagel3;
+
+	unsigned char m_gasmbuschannel;
+	std::string m_gasprefix;
+	std::string m_gastimestamp;
+	double m_gasclockskew;
+	time_t m_gasoktime;
+
+	void Init();
+	bool MatchLine();
+	void ParseData(const unsigned char *pData, const int Len, const bool disable_crc, int ratelimit);
+
 	bool CheckCRC();
 };
-
