@@ -7,18 +7,18 @@
 class P1MeterTCP: public P1MeterBase
 {
 public:
-	P1MeterTCP(const int ID, const std::string &IPAddress, const unsigned short usIPPort, unsigned char disable_crc);
+	P1MeterTCP(const int ID, const std::string &IPAddress, const unsigned short usIPPort, const bool disable_crc, const int ratelimit);
 	~P1MeterTCP(void);
 
 	void write(const char *data, size_t size);
 	bool isConnected(){ return m_socket!= INVALID_SOCKET; };
 	bool WriteToHardware(const char *pdata, const unsigned char length);
-	unsigned char m_DisableCRC;
 public:
 	// signals
 	boost::signals2::signal<void()>	sDisconnected;
 private:
 	int m_retrycntr;
+	int m_ratelimit;
 	bool StartHardware();
 	bool StopHardware();
 	void disconnect();
@@ -33,4 +33,3 @@ protected:
 	sockaddr_in m_addr;
 	int m_socket;
 };
-
