@@ -78,7 +78,7 @@ void CRFLinkSerial::Do_Work()
 				} else
 				if (atime - m_LastReceivedTime > 50) {
 				//Timeout
-				_log.Log(LOG_ERROR, "RFLink: Nothing received for more then 50 seconds, restarting...");
+				_log.Log(LOG_ERROR, "RFLink: Nothing received for more than 50 seconds, restarting...");
 				m_retrycntr = 0;
 				m_LastReceivedTime = atime;
 				terminate();
@@ -96,14 +96,14 @@ void CRFLinkSerial::Do_Work()
 					time_t atime = mytime(NULL);
 					//Send ping (keep alive)
 					//_log.Log(LOG_STATUS, "RFLink: t1=%d t3=%d", atime, m_LastReceivedTime);
-					if (atime - m_LastReceivedTime > 50) {
+					if (difftime(atime,m_LastReceivedTime) > 50) {
 						//Receive Timeout
 						//_log.Log(LOG_STATUS, "RFLink: ping50...");
 						write("10;PING;\n");
 						m_retrycntr = 0;
 						m_LastReceivedTime = atime;
 					} else {
-						if (atime - m_LastReceivedTime > 25) {
+						if (difftime(atime,m_LastReceivedTime) > 25) {
 						   //_log.Log(LOG_STATUS, "RFLink: ping25...");
 						   write("10;PING;\n");
 						}
