@@ -11,6 +11,8 @@ public:
 	bool WriteToHardware(const char *pdata, const unsigned char length);
 	std::string GetForecastURL();
 private:
+	bool m_bForceSingleStation;
+	bool m_bFirstTime;
 	std::string m_APIKey;
 	std::string m_Location;
 	volatile bool m_stoprequested;
