@@ -19,8 +19,8 @@ Domoticz and the lights need to be in the same network/subnet
 */
 
 #ifdef _DEBUG
-	//#define DEBUG_YeeLightR
-	//#define DEBUG_YeeLightW
+//#define DEBUG_YeeLightR
+//#define DEBUG_YeeLightW
 #endif
 
 #ifdef DEBUG_YeeLightW
@@ -100,7 +100,7 @@ void Yeelight::Do_Work()
 
 	boost::asio::io_service io_service;
 	udp_server server(io_service, m_HwdID);
-	int sec_counter = Limitless_POLL_INTERVAL-5;
+	int sec_counter = Limitless_POLL_INTERVAL - 5;
 	while (!m_stoprequested)
 	{
 		sleep_seconds(1);
@@ -168,11 +168,12 @@ void Yeelight::InsertUpdateSwitch(const std::string &nodeID, const std::string &
 		int value = atoi(yeelightBright.c_str());
 		if ((bIsOn != tIsOn) || (value != lastLevel))
 		{
-			int cmd = light1_sOn;
-			int level = 100;
+			int cmd = Limitless_LedOn;
 			if (!bIsOn) {
-				cmd = light1_sOff;
-				level = 0;
+				cmd = Limitless_LedOff;
+			}
+			if (value != lastLevel) {
+				cmd = Limitless_SetBrightnessLevel;
 			}
 			_tLimitlessLights ycmd;
 			ycmd.len = sizeof(_tLimitlessLights) - 1;
@@ -196,6 +197,7 @@ bool Yeelight::WriteToHardware(const char *pdata, const unsigned char length)
 	std::vector<std::vector<std::string> > result;
 	unsigned long lID;
 	char szTmp[50];
+	bool sendOnFirst = false;
 
 	if (pLed->id == 1)
 		sprintf(szTmp, "%d", 1);
@@ -227,12 +229,13 @@ bool Yeelight::WriteToHardware(const char *pdata, const unsigned char length)
 	catch (const std::exception &e)
 	{
 		_log.Log(LOG_ERROR, "YeeLight: Exception: %s", e.what());
+		return false;
 	}
 
-	std::string message = "{\"id\":1,\"method\":\"toggle\",\"params\":[]}\r\n";
-
+	std::string message = "";
 	char request[1024];
 	size_t request_length;
+	std::stringstream ss;
 
 	switch (pLed->command)
 	{
@@ -242,68 +245,126 @@ bool Yeelight::WriteToHardware(const char *pdata, const unsigned char length)
 	case Limitless_LedOff:
 		message = "{\"id\":1,\"method\":\"set_power\",\"params\":[\"off\", \"smooth\", 500]}\r\n";
 		break;
-	case Limitless_SetColorToWhite:
-		message = "{\"id\":1,\"method\":\"set_power\",\"params\":[\"on\", \"smooth\", 500]}\r\n";
-		strcpy(request, message.c_str());
-		request_length = strlen(request);
-		boost::asio::write(sendSocket, boost::asio::buffer(request, request_length));
-		sleep_milliseconds(200);
-		message = "{\"id\":1,\"method\":\"set_rgb\",\"params\":[16777215, \"smooth\", 500]}\r\n";
-		break;
-	case Limitless_SetBrightnessLevel:
-		{
-			message = "{\"id\":1,\"method\":\"set_power\",\"params\":[\"on\", \"smooth\", 500]}\r\n";
-			strcpy(request, message.c_str());
-			request_length = strlen(request);
-			boost::asio::write(sendSocket, boost::asio::buffer(request, request_length));
-			sleep_milliseconds(200);
-			std::stringstream ss;
-			ss << "{\"id\":1,\"method\":\"set_bright\",\"params\":[" << int(pLed->value) << ", \"smooth\", 500]}\r\n";
-			message = ss.str();
+	case Limitless_LedNight:
+		if (pLed->subtype == sTypeLimitlessRGBW) {
+			message = "{\"id\":1,\"method\":\"set_scene\", \"params\": [\"color\", 16750848, 1]}\r\n";
+		}
+		else {
+			message = "{\"id\":1,\"method\":\"set_bright\",\"params\":[1, \"smooth\", 500]}\r\n";
 		}
 		break;
-	case Limitless_SetRGBColour:
-		{
-			message = "{\"id\":1,\"method\":\"set_power\",\"params\":[\"on\", \"smooth\", 500]}\r\n";
-			strcpy(request, message.c_str());
-			request_length = strlen(request);
-			boost::asio::write(sendSocket, boost::asio::buffer(request, request_length));
-			sleep_milliseconds(200);
-			float cHue = (359.0f / 255.0f)*float(pLed->value);//hue given was in range of 0-255
-			//message = "{\"id\":1,\"method\":\"set_hsv\",\"params\":[" + std::to_string(cHue) + ", 100, \"smooth\", 2000]}\r\n";
-			std::stringstream ss;
+	case Limitless_LedFull:
+		message = "{\"id\":1,\"method\":\"set_bright\",\"params\":[100, \"smooth\", 500]}\r\n";
+		break;
+	case Limitless_BrightnessUp:
+		message = "{\"id\":1,\"method\":\"set_adjust\",\"params\":[\"increase\", \"bright\"]}\r\n";
+		break;
+	case Limitless_BrightnessDown:
+		message = "{\"id\":1,\"method\":\"set_adjust\",\"params\":[\"decrease\", \"bright\"]}\r\n";
+		break;
+	case Limitless_ColorTempUp:
+		message = "{\"id\":1,\"method\":\"set_adjust\",\"params\":[\"increase\", \"ct\"]}\r\n";
+		break;
+	case Limitless_ColorTempDown:
+		message = "{\"id\":1,\"method\":\"set_adjust\",\"params\":[\"decrease\", \"ct\"]}\r\n";
+		break;
+	case Limitless_SetColorToWhite:
+		sendOnFirst = true;
+		if (pLed->subtype == sTypeLimitlessRGBW) {
+			message = "{\"id\":1,\"method\":\"set_rgb\",\"params\":[16777215, \"smooth\", 500]}\r\n";
+		}
+		else {
+			message = "{\"id\":1,\"method\":\"set_bright\",\"params\":[100, \"smooth\", 500]}\r\n";
+		}
+		break;
+	case Limitless_SetBrightnessLevel:
+		sendOnFirst = true;
+		ss << "{\"id\":1,\"method\":\"set_bright\",\"params\":[" << int(pLed->value) << ", \"smooth\", 500]}\r\n";
+		message = ss.str();
+		break;
+	case Limitless_SetRGBColour: {
+			sendOnFirst = true;
+			float cHue = (359.0f / 255.0f)*float(pLed->value); // hue given was in range of 0-255
 			ss << "{\"id\":1,\"method\":\"set_hsv\",\"params\":[" << cHue << ", 100, \"smooth\", 2000]}\r\n";
 			message = ss.str();
 		}
 		break;
+	case Limitless_SetBrightUp:
+		message = "{\"id\":1,\"method\":\"set_adjust\",\"params\":[\"increase\", \"bright\"]}\r\n";
+		break;
+	case Limitless_SetBrightDown:
+		message = "{\"id\":1,\"method\":\"set_adjust\",\"params\":[\"decrease\", \"bright\"]}\r\n";
+		break;
+	case Limitless_WarmWhiteIncrease:
+		//message = "{\"id\":1,\"method\":\"set_adjust\",\"params\":[\"increase\", \"bright\"]}\r\n";
+		message = "{\"id\":1,\"method\":\"set_ct_abx\",\"params\":[3500, \"smooth\", 500]}\r\n";
+		break;
+	case Limitless_CoolWhiteIncrease:
+		//message = "{\"id\":1,\"method\":\"set_adjust\",\"params\":[\"decrease\", \"bright\"]}\r\n";
+		message = "{\"id\":1,\"method\":\"set_ct_abx\",\"params\":[6000, \"smooth\", 500]}\r\n";
+		break;
+	case Limitless_NightMode:
+		if (pLed->subtype == sTypeLimitlessRGBW) {
+			message = "{\"id\":1,\"method\":\"set_scene\", \"params\": [\"color\", 16750848, 1]}\r\n";
+		}
+		else {
+			message = "{\"id\":1,\"method\":\"set_bright\",\"params\":[1, \"smooth\", 500]}\r\n";
+		}
+		break;
+	case Limitless_FullBrightness: {
+			sendOnFirst = true;
+			int value = pLed->value;
+			ss << "{\"id\":1,\"method\":\"set_bright\",\"params\":[100, \"smooth\", 500]}\r\n";
+			message = ss.str();
+		}
+		break;
+	case Limitless_DiscoMode:
+		sendOnFirst = true;
+		// simulate strobe effect - at time of writing, minimum timing allowed by Yeelight is 50ms
+		_log.Log(LOG_STATUS, "Yeelight: Disco Mode - simulate strobe effect, if you have a suggestion for what it should do, please post on the Domoticz forum");
+		message = "{\"id\":1,\"method\":\"start_cf\",\"params\":[ 50, 0, \"";
+		message += "50, 2, 5000, 100, ";
+		message += "50, 2, 5000, 1\"]}\r\n";
+		break;
+	case Limitless_DiscoSpeedFasterLong:
+		_log.Log(LOG_STATUS, "Yeelight: Exclude Lamp - This command is unhandled, if you have a suggestion for what it should do, please post on the Domoticz forum");
+		break;
 	default:
-		_log.Log(LOG_STATUS, "YeeLight: Unhandled WriteToHardware command: %d", command);
+		_log.Log(LOG_STATUS, "YeeLight: Unhandled WriteToHardware command: %d - if you have a suggestion for what it should do, please post on the Domoticz forum", command);
 		break;
 	}
 
+	if (message == "") {
+		return false;
+	}
+
+	if (sendOnFirst) {
+		strcpy(request, "{\"id\":1,\"method\":\"set_power\",\"params\":[\"on\", \"smooth\", 500]}\r\n");
+		request_length = strlen(request);
+		boost::asio::write(sendSocket, boost::asio::buffer(request, request_length));
+		sleep_milliseconds(50);
+	}
 
 	strcpy(request, message.c_str());
 	request_length = strlen(request);
 	boost::asio::write(sendSocket, boost::asio::buffer(request, request_length));
-
-	//_log.Log(LOG_STATUS, "message is sent..................");
-	sleep_milliseconds(200);
-
+	sleep_milliseconds(50);
 	return true;
 }
 
 
-boost::array<char, 4096> recv_buffer_;
+boost::array<char, 1024> recv_buffer_;
 int hardwareId;
 
 Yeelight::udp_server::udp_server(boost::asio::io_service& io_service, int m_HwdID)
-	: socket_(io_service, boost::asio::ip::udp::endpoint(boost::asio::ip::udp::v4(), 1982))
+	: socket_(io_service, boost::asio::ip::udp::endpoint(boost::asio::ip::udp::v4(), 0))
 {
 	socket_.set_option(boost::asio::ip::udp::socket::reuse_address(true));
 	socket_.set_option(boost::asio::socket_base::broadcast(true));
 	hardwareId = m_HwdID;
 }
 
+
 void Yeelight::udp_server::start_send()
 {
 	std::string testMessage = "M-SEARCH * HTTP/1.1\r\nHOST: 239.255.255.250:1982\r\nMAN: \"ssdp:discover\"\r\nST: wifi_bulb";
@@ -322,10 +383,12 @@ void Yeelight::udp_server::start_receive()
 	std::string szData = ReadFile("E:\\YeeLight_receive.txt");
 	HandleIncoming(szData);
 #endif
+	// only allow one response from each ip per run
+	std::vector<std::string> receivedip;
+
 	while (socket_.available() > 0) {
 		socket_.receive_from(boost::asio::buffer(recv_buffer_), remote_endpoint_);
-		//_log.Log(LOG_STATUS, "data received......");
-		HandleIncoming(recv_buffer_.data());
+		HandleIncoming(recv_buffer_.data(), receivedip);
 	}
 }
 
@@ -342,7 +405,8 @@ bool YeeLightGetTag(const std::string &InputString, const std::string &Tag, std:
 	return true;
 }
 
-bool Yeelight::udp_server::HandleIncoming(const std::string &szData)
+
+bool Yeelight::udp_server::HandleIncoming(const std::string &szData, std::vector<std::string> &receivedip)
 {
 	std::string receivedString(szData);
 	//_log.Log(LOG_STATUS, receivedString.c_str());
@@ -362,8 +426,16 @@ bool Yeelight::udp_server::HandleIncoming(const std::string &szData)
 	std::string dataString = receivedString.substr(pos, pos1);
 
 	std::string yeelightLocation = dataString.c_str();
-	//_log.Log(LOG_STATUS, "LLocation: %s",yeelightLocation.c_str());
-
+	// check if we have received this ip already
+	size_t i;
+	for (i = 0; i < receivedip.size(); i++) {
+		if (std::strcmp(receivedip[i].c_str(), yeelightLocation.c_str()) == 0) {
+			//_log.Log(LOG_STATUS, "Already received: %s", yeelightLocation.c_str());
+			return false;
+		}
+	}
+	receivedip.push_back(yeelightLocation);
+	//_log.Log(LOG_STATUS, "Location: %s", yeelightLocation.c_str());
 	std::string yeelightId;
 	if (!YeeLightGetTag(szData, "id: ", yeelightId))
 		return false;
@@ -396,7 +468,7 @@ bool Yeelight::udp_server::HandleIncoming(const std::string &szData)
 	if (yeelightModel == "mono") {
 		yeelightName = "YeeLight LED (Mono)";
 	}
-	else if (yeelightModel == "color") {
+	else if ((yeelightModel == "color") || (yeelightModel == "stripe")) {
 		yeelightName = "YeeLight LED (Color)";
 		sType = sTypeLimitlessRGBW;
 	}
@@ -428,9 +500,7 @@ namespace http {
 			int HwdID = atoi(idx.c_str());
 
 			Yeelight yeelight(HwdID);
-			yeelight.InsertUpdateSwitch("123", sname, (stype=="0")? sTypeLimitlessWhite:sTypeLimitlessRGBW, sipaddress, false, "0", "0");
+			yeelight.InsertUpdateSwitch("123", sname, (stype == "0") ? sTypeLimitlessWhite : sTypeLimitlessRGBW, sipaddress, false, "0", "0");
 		}
 	}
 }
-
-
