@@ -108,7 +108,7 @@ void ZWaveBase::Do_Work()
 			if (m_bControllerCommandInProgress==true)
 			{
 				time_t atime=mytime(NULL);
-				time_t tdiff=atime-m_ControllerCommandStartTime;
+				double tdiff=difftime(atime,m_ControllerCommandStartTime);
 				if (tdiff>=CONTROLLER_COMMAND_TIMEOUT)
 				{
 					_log.Log(LOG_STATUS,"ZWave: Stopping Controller command (Timeout!)");
@@ -176,7 +176,10 @@ bool ZWaveBase::IsNodeRGBW(const unsigned int homeID, const int nodeID)
 	if (result.size() < 1)
 		return false;
 	std::string ProductDescription = result[0][0];
-	return (ProductDescription.find("FGRGBWM441") != std::string::npos);
+	return (
+		(ProductDescription.find("FGRGBWM441") != std::string::npos)
+		|| (ProductDescription.find("ZMNHWD") != std::string::npos)
+		);
 }
 
 void ZWaveBase::SendSwitchIfNotExists(const _tZWaveDevice *pDevice)
@@ -184,7 +187,7 @@ void ZWaveBase::SendSwitchIfNotExists(const _tZWaveDevice *pDevice)
 	if (
 		(pDevice->devType!=ZDTYPE_SWITCH_NORMAL)&&
 		(pDevice->devType != ZDTYPE_SWITCH_DIMMER) &&
-		(pDevice->devType != ZDTYPE_SWITCH_FGRGBWM441)&&
+		(pDevice->devType != ZDTYPE_SWITCH_RGBW)&&
 		(pDevice->devType != ZDTYPE_SWITCH_COLOR)
 		)
 		return; //only for switches
@@ -195,7 +198,7 @@ void ZWaveBase::SendSwitchIfNotExists(const _tZWaveDevice *pDevice)
 		BatLevel = pDevice->batValue;
 	}
 
-	if ((pDevice->devType == ZDTYPE_SWITCH_FGRGBWM441) || (pDevice->devType == ZDTYPE_SWITCH_COLOR))
+	if ((pDevice->devType == ZDTYPE_SWITCH_RGBW) || (pDevice->devType == ZDTYPE_SWITCH_COLOR))
 	{
 		unsigned char ID1 = 0;
 		unsigned char ID2 = 0;
@@ -280,7 +283,7 @@ void ZWaveBase::SendSwitchIfNotExists(const _tZWaveDevice *pDevice)
 		int level = pDevice->intvalue;
 
 		// Simple on/off device, make sure we only have 0 or 255
-		if (pDevice->devType == ZDTYPE_SWITCH_NORMAL)
+		if ((pDevice->devType == ZDTYPE_SWITCH_NORMAL)|| (pDevice->devType == ZDTYPE_CENTRAL_SCENE))
 			level = (level == 0) ? 0 : 255;
 
 		// Now check the values
@@ -346,7 +349,7 @@ void ZWaveBase::SendDevice2Domoticz(const _tZWaveDevice *pDevice)
 		BatLevel = pDevice->batValue;
 	}
 
-	if ((pDevice->devType==ZDTYPE_SWITCH_NORMAL)||(pDevice->devType==ZDTYPE_SWITCH_DIMMER))
+	if ((pDevice->devType==ZDTYPE_SWITCH_NORMAL)||(pDevice->devType==ZDTYPE_SWITCH_DIMMER)||(pDevice->devType== ZDTYPE_CENTRAL_SCENE))
 	{
 		//Send as pTypeGeneralSwitch, sSwitchGeneralSwitch
 		_tGeneralSwitch gswitch;
@@ -359,7 +362,7 @@ void ZWaveBase::SendDevice2Domoticz(const _tZWaveDevice *pDevice)
 		int level = pDevice->intvalue;
 
 		// Simple on/off device, make sure we only have 0 or 255
-		if (pDevice->devType == ZDTYPE_SWITCH_NORMAL)
+		if ((pDevice->devType == ZDTYPE_SWITCH_NORMAL)|| (pDevice->devType == ZDTYPE_CENTRAL_SCENE))
 			level = (level == 0) ? 0 : 255;
 
 		// Now check the values
@@ -384,7 +387,7 @@ void ZWaveBase::SendDevice2Domoticz(const _tZWaveDevice *pDevice)
 		sDecodeRXMessage(this, (const unsigned char *)&gswitch, NULL, BatLevel);
 		return;
 	}
-	else if ((pDevice->devType == ZDTYPE_SWITCH_FGRGBWM441) || (pDevice->devType == ZDTYPE_SWITCH_COLOR))
+	else if ((pDevice->devType == ZDTYPE_SWITCH_RGBW) || (pDevice->devType == ZDTYPE_SWITCH_COLOR))
 	{
 		unsigned char ID1 = 0;
 		unsigned char ID2 = 0;
@@ -697,11 +700,13 @@ void ZWaveBase::SendDevice2Domoticz(const _tZWaveDevice *pDevice)
 	}
 	else if (pDevice->devType == ZDTYPE_SENSOR_WATER)
 	{
-		SendMeterSensor(ID3, ID4, BatLevel, pDevice->floatValue,"Water");
+		uint16_t NodeID = (ID3 << 8) | ID4;
+		SendRainSensor(NodeID, BatLevel, pDevice->floatValue*1000.0f, "Water");
+		//SendMeterSensor(ID3, ID4, BatLevel, pDevice->floatValue,"Water");
 	}
 	else if (pDevice->devType == ZDTYPE_SENSOR_CO2)
 	{
-		SendAirQualitySensor(ID3, ID4, BatLevel, int(pDevice->floatValue), "CO2 Sensor");
+		SendAirQualitySensor(ID3, (uint8_t)pDevice->orgInstanceID, BatLevel, int(pDevice->floatValue), "CO2 Sensor");
 	}
 	else if (pDevice->devType == ZDTYPE_SENSOR_MOISTURE)
 	{
@@ -753,6 +758,21 @@ void ZWaveBase::SendDevice2Domoticz(const _tZWaveDevice *pDevice)
 		gDevice.intval2 = pDevice->intvalue;
 		sDecodeRXMessage(this, (const unsigned char *)&gDevice, "Thermostat Fan Mode", BatLevel);
 	}
+	else if (pDevice->devType == ZDTYPE_ALARM)
+	{
+		_tGeneralDevice gDevice;
+		gDevice.subtype = sTypeZWaveAlarm;
+		gDevice.id = pDevice->Alarm_Type;
+		gDevice.intval1 = (int)(ID1 << 24) | (ID2 << 16) | (ID3 << 8) | ID4;
+		gDevice.intval2 = pDevice->intvalue;
+		char szTmp[100];
+		sprintf(szTmp, "Alarm Type: %s %d(0x%02X)", pDevice->label.c_str(), pDevice->Alarm_Type, pDevice->Alarm_Type);
+		sDecodeRXMessage(this, (const unsigned char *)&gDevice, szTmp, BatLevel);
+	}
+	else if (pDevice->devType == ZDTYPE_SENSOR_CUSTOM)
+	{
+		SendCustomSensor(ID3, ID4, BatLevel, pDevice->floatValue, pDevice->label, pDevice->custom_label);
+	}
 }
 
 ZWaveBase::_tZWaveDevice* ZWaveBase::FindDevice(const int nodeID, const int instanceID, const int indexID)
@@ -991,7 +1011,7 @@ bool ZWaveBase::WriteToHardware(const char *pdata, const unsigned char length)
 		int nodeID = (ID2 << 8) | ID3;
 		int instanceID = ID4;
 		int indexID = ID1;
-		pDevice = FindDevice(nodeID, instanceID, indexID, ZDTYPE_SWITCH_FGRGBWM441);
+		pDevice = FindDevice(nodeID, instanceID, indexID, ZDTYPE_SWITCH_RGBW);
 		if (pDevice)
 		{
 			int svalue = 0;
@@ -1151,7 +1171,7 @@ void ZWaveBase::ForceUpdateForNodeDevices(const unsigned int homeID, const int n
 				{
 					if (IsNodeRGBW(homeID, nodeID))
 					{
-						zdevice.devType = ZDTYPE_SWITCH_FGRGBWM441;
+						zdevice.devType = ZDTYPE_SWITCH_RGBW;
 						zdevice.instanceID = 100;
 						SendDevice2Domoticz(&zdevice);
 					}
