@@ -1337,6 +1337,119 @@ std::string COpenZWaveControlPanel::DoAdminCommand(const std::string &fun, const
 	return "OK";
 }
 
+std::string COpenZWaveControlPanel::DoSceneCommand(const std::string &fun, const std::string &arg1, const std::string &arg2, const std::string &arg3)
+{
+	TiXmlDocument doc;
+	char str[16];
+	string s;
+	int cnt;
+	int i;
+	uint8 sid;
+	TiXmlDeclaration* decl = new TiXmlDeclaration("1.0", "utf-8", "");
+	doc.LinkEndChild(decl);
+	TiXmlElement* scenesElement = new TiXmlElement("scenes");
+	doc.LinkEndChild(scenesElement);
+
+	if (fun == "create") {
+		sid = Manager::Get()->CreateScene();
+		if (sid == 0) {
+			fprintf(stderr, "sid = 0, out of scene ids\n");
+			return "";
+		}
+	}
+	if ((fun == "values") ||
+		(fun == "label") ||
+		(fun == "delete") ||
+		(fun == "execute") ||
+		(fun == "addvalue") ||
+		(fun == "update") ||
+		(fun == "remove")) {
+		sid = (uint8)atoi(arg1.c_str());
+		if (fun == "delete")
+			Manager::Get()->RemoveScene(sid);
+		else if (fun == "execute")
+			Manager::Get()->ActivateScene(sid);
+		else if (fun == "label")
+			Manager::Get()->SetSceneLabel(sid, string(arg2));
+		if ((fun == "addvalue") ||
+			(fun == "update") ||
+			(fun == "remove")) {
+			MyValue *val = MyNode::lookup(string(arg2));
+			if (val != NULL) {
+				if (fun == "addvalue") {
+					if (!Manager::Get()->AddSceneValue(sid, val->getId(), string(arg3)))
+						fprintf(stderr, "AddSceneValue failure\n");
+				}
+				else if (fun == "update") {
+					if (!Manager::Get()->SetSceneValue(sid, val->getId(), string(arg3)))
+						fprintf(stderr, "SetSceneValue failure\n");
+				}
+				else if (fun == "remove") {
+					if (!Manager::Get()->RemoveSceneValue(sid, val->getId()))
+						fprintf(stderr, "RemoveSceneValue failure\n");
+				}
+			}
+		}
+	}
+	if ((fun=="load") ||
+		(fun=="create") ||
+		(fun=="label") ||
+		(fun=="delete")) { // send all sceneids
+		uint8 *sptr;
+		cnt = Manager::Get()->GetAllScenes(&sptr);
+		scenesElement->SetAttribute("sceneid", cnt);
+		for (i = 0; i < cnt; i++) {
+			TiXmlElement* sceneElement = new TiXmlElement("sceneid");
+			snprintf(str, sizeof(str), "%d", sptr[i]);
+			sceneElement->SetAttribute("id", str);
+			s = Manager::Get()->GetSceneLabel(sptr[i]);
+			sceneElement->SetAttribute("label", s.c_str());
+			scenesElement->LinkEndChild(sceneElement);
+		}
+		delete[] sptr;
+	}
+	if ((fun=="values") ||
+		(fun=="addvalue") ||
+		(fun=="remove") ||
+		(fun=="update")) {
+		vector<ValueID> vids;
+		cnt = Manager::Get()->SceneGetValues(sid, &vids);
+		scenesElement->SetAttribute("scenevalue", cnt);
+		for (vector<ValueID>::iterator it = vids.begin(); it != vids.end(); it++) {
+			TiXmlElement* valueElement = new TiXmlElement("scenevalue");
+			valueElement->SetAttribute("id", sid);
+			snprintf(str, sizeof(str), "0x%x", (*it).GetHomeId());
+			valueElement->SetAttribute("home", str);
+			valueElement->SetAttribute("node", (*it).GetNodeId());
+			valueElement->SetAttribute("class", cclassStr((*it).GetCommandClassId()));
+			valueElement->SetAttribute("instance", (*it).GetInstance());
+			valueElement->SetAttribute("index", (*it).GetIndex());
+			valueElement->SetAttribute("type", valueTypeStr((*it).GetType()));
+			valueElement->SetAttribute("genre", valueGenreStr((*it).GetGenre()));
+			valueElement->SetAttribute("label", Manager::Get()->GetValueLabel(*it).c_str());
+			valueElement->SetAttribute("units", Manager::Get()->GetValueUnits(*it).c_str());
+			Manager::Get()->SceneGetValueAsString(sid, *it, &s);
+			TiXmlText *textElement = new TiXmlText(s.c_str());
+			valueElement->LinkEndChild(textElement);
+			scenesElement->LinkEndChild(valueElement);
+		}
+	}
+	char fntemp[200];
+	sprintf(fntemp, "%sozwcp.scenes.XXXXXX", szUserDataFolder.c_str());
+	doc.SaveFile(fntemp);
+
+	std::string retstring = "";
+	std::ifstream testFile(fntemp, std::ios::binary);
+	std::vector<char> fileContents((std::istreambuf_iterator<char>(testFile)),
+		std::istreambuf_iterator<char>());
+	if (fileContents.size() > 0)
+	{
+		retstring.insert(retstring.begin(), fileContents.begin(), fileContents.end());
+	}
+
+	return retstring;
+}
+
 std::string COpenZWaveControlPanel::DoNodeChange(const std::string &fun, const int node_id, const std::string &svalue)
 {
 	if (fun == "nam") { /* Node naming */
