@@ -7,16 +7,19 @@
 
 enum _eLogLevel
 {
-	LOG_NORM=0,
-	LOG_ERROR,
-	LOG_STATUS,
+	LOG_ERROR = 0,
+	LOG_STATUS = 1,
+	LOG_NORM = 2,
+	LOG_TRACE = 3
 };
 
 enum _eLogFileVerboseLevel
 {
-	VBL_ALL=0,
-	VBL_STATUS_ERROR,
-	VBL_ERROR,
+	VBL_ERROR = 0,
+	VBL_STATUS_ERROR = 1,
+	VBL_ALL = 2,
+	VBL_TRACE,
+
 };
 
 class CLogger
@@ -37,7 +40,6 @@ public:
 	void SetVerboseLevel(_eLogFileVerboseLevel vLevel);
 
 	void Log(const _eLogLevel level, const char* logline, ...);
-	void LogNoLF(const _eLogLevel level, const char* logline, ...);
 
 	void LogSequenceStart();
 	void LogSequenceAdd(const char* logline);
@@ -45,7 +47,16 @@ public:
 	void LogSequenceEnd(const _eLogLevel level);
 
 	void EnableLogTimestamps(const bool bEnableTimestamps);
+	bool IsLogTimestampsEnabled();
 
+	void SetFilterString(std::string &Filter);
+	bool isTraceEnabled();
+	bool TestFilter(const char *cbuffer);
+	void setLogVerboseLevel(int LogLevel);
+	void SetLogPreference(std::string LogFilter, std::string LogFileName, std::string LogLevel);
+	void GetLogPreference();
+	void SetLogDebug(bool debug);
+	bool GetLogDebug();
 	void ForwardErrorsToNotificationSystem(const bool bDoForward);
 
 	std::list<_tLogLineStruct> GetLog(const _eLogLevel lType);
@@ -65,6 +76,10 @@ private:
 	bool m_bEnableErrorsToNotificationSystem;
 	time_t m_LastLogNotificationsSend;
 	std::stringstream m_sequencestring;
+	std::string FilterString;
+	std::vector<std::string> FilterStringList;	//contain the list of filtered words
+	std::vector<std::string> KeepStringList;	//contain the list of  words to be kept
 	_eLogFileVerboseLevel m_verbose_level;
+	bool m_debug;
 };
 extern CLogger _log;
