@@ -10,6 +10,8 @@
 #include "../webserver/cWebem.h"
 #include "../json/json.h"
 #include "boost/date_time/gregorian/gregorian.hpp"
+#define __STDC_FORMAT_MACROS
+#include <inttypes.h>
 
 CScheduler::CScheduler(void)
 {
@@ -356,7 +358,7 @@ void CScheduler::SetSunRiseSetTimers(const std::string &sSunRise, const std::str
 	bool bReloadSchedules = false;
 	{	//needed private scope for the lock
 		boost::lock_guard<boost::mutex> l(m_mutex);
-		unsigned char hour, min, sec;
+		int hour, min, sec;
 
 		time_t temptime;
 		time_t atime = mytime(NULL);
@@ -367,10 +369,8 @@ void CScheduler::SetSunRiseSetTimers(const std::string &sSunRise, const std::str
 		min = atoi(sSunRise.substr(3, 2).c_str());
 		sec = atoi(sSunRise.substr(6, 2).c_str());
 
-		ltime.tm_hour = hour;
-		ltime.tm_min = min;
-		ltime.tm_sec = sec;
-		temptime = mktime(&ltime);
+		struct tm tm1;
+		constructTime(temptime,tm1,ltime.tm_year+1900,ltime.tm_mon+1,ltime.tm_mday,hour,min,sec,ltime.tm_isdst);
 		if ((m_tSunRise != temptime) && (temptime != 0))
 		{
 			if (m_tSunRise == 0)
@@ -382,10 +382,7 @@ void CScheduler::SetSunRiseSetTimers(const std::string &sSunRise, const std::str
 		min = atoi(sSunSet.substr(3, 2).c_str());
 		sec = atoi(sSunSet.substr(6, 2).c_str());
 
-		ltime.tm_hour = hour;
-		ltime.tm_min = min;
-		ltime.tm_sec = sec;
-		temptime = mktime(&ltime);
+		constructTime(temptime,tm1,ltime.tm_year+1900,ltime.tm_mon+1,ltime.tm_mday,hour,min,sec,ltime.tm_isdst);
 		if ((m_tSunSet != temptime) && (temptime != 0))
 		{
 			if (m_tSunSet == 0)
@@ -403,7 +400,9 @@ bool CScheduler::AdjustScheduleItem(tScheduleItem *pItem, bool bForceAddDay)
 	time_t rtime = atime;
 	struct tm ltime;
 	localtime_r(&atime, &ltime);
-	ltime.tm_sec = 0;
+	int isdst = ltime.tm_isdst;
+	struct tm tm1;
+	tm1.tm_isdst = -1;
 
 	unsigned long HourMinuteOffset = (pItem->startHour * 3600) + (pItem->startMin * 60);
 
@@ -429,18 +428,11 @@ bool CScheduler::AdjustScheduleItem(tScheduleItem *pItem, bool bForceAddDay)
 		(pItem->timerType == TTYPE_WEEKSODD) ||
 		(pItem->timerType == TTYPE_WEEKSEVEN))
 	{
-		ltime.tm_hour = pItem->startHour;
-		ltime.tm_min = pItem->startMin;
-		rtime = mktime(&ltime) + (roffset * 60);
+		constructTime(rtime,tm1,ltime.tm_year+1900,ltime.tm_mon+1,ltime.tm_mday,pItem->startHour,pItem->startMin,roffset*60,isdst);
 	}
 	else if (pItem->timerType == TTYPE_FIXEDDATETIME)
 	{
-		ltime.tm_year = pItem->startYear - 1900;
-		ltime.tm_mon = pItem->startMonth - 1;
-		ltime.tm_mday = pItem->startDay;
-		ltime.tm_hour = pItem->startHour;
-		ltime.tm_min = pItem->startMin;
-		rtime = mktime(&ltime) + (roffset * 60);
+		constructTime(rtime,tm1,pItem->startYear,pItem->startMonth,pItem->startDay,pItem->startHour,pItem->startMin,roffset*60,isdst);
 		if (rtime < atime)
 			return false; //past date/time
 		pItem->startTime = rtime;
@@ -472,38 +464,20 @@ bool CScheduler::AdjustScheduleItem(tScheduleItem *pItem, bool bForceAddDay)
 	}
 	else if (pItem->timerType == TTYPE_MONTHLY)
 	{
-		ltime.tm_mday = pItem->MDay;
-		ltime.tm_hour = pItem->startHour;
-		ltime.tm_min = pItem->startMin;
-		//if mday exceeds max days in month, find next month with this amount of days
-		while(ltime.tm_mday > boost::gregorian::gregorian_calendar::end_of_month_day(ltime.tm_year + 1900, ltime.tm_mon + 1))
+		constructTime(rtime,tm1,ltime.tm_year+1900,ltime.tm_mon+1,pItem->MDay,pItem->startHour,pItem->startMin,0,isdst);
+
+		while ( (rtime < atime) || (tm1.tm_mday != pItem->MDay) ) // past date/time OR mday exceeds max days in month
 		{
 			ltime.tm_mon++;
-			if (ltime.tm_mon > 11)
-			{
-				ltime.tm_mon = 0;
-				ltime.tm_year++;
-			}
-		}
-		rtime = mktime(&ltime) + (roffset * 60);
-		if (rtime < atime) //past date/time
-		{
-			//schedule for next month
-			boost::gregorian::month_iterator m_itr(boost::gregorian::date(ltime.tm_year + 1900, ltime.tm_mon + 1, ltime.tm_mday));
-			++m_itr;
-			ltime.tm_mon = m_itr->month() - 1;
-			ltime.tm_year = m_itr->year() - 1900;
-			rtime = mktime(&ltime) + (roffset * 60);
+			constructTime(rtime,tm1,ltime.tm_year+1900,ltime.tm_mon+1,pItem->MDay,pItem->startHour,pItem->startMin,0,isdst);
 		}
 
+		rtime += roffset * 60; // add randomness
 		pItem->startTime = rtime;
 		return true;
 	}
 	else if (pItem->timerType == TTYPE_MONTHLY_WD)
 	{
-		ltime.tm_hour = pItem->startHour;
-		ltime.tm_min = pItem->startMin;
-
 		//pItem->Days: mon=1 .. sat=32, sun=64
 		//convert to : sun=0, mon=1 .. sat=6
 		int daynum = (int)log2(pItem->Days) + 1;
@@ -517,53 +491,45 @@ bool CScheduler::AdjustScheduleItem(tScheduleItem *pItem, bool bForceAddDay)
 		nth_dow ndm(Occurence, Day, Month);
 		boost::gregorian::date d = ndm.get_date(ltime.tm_year + 1900);
 
-		ltime.tm_mday = d.day();
-		rtime = mktime(&ltime) + (roffset * 60);
+		constructTime(rtime,tm1,ltime.tm_year+1900,ltime.tm_mon+1,d.day(),pItem->startHour,pItem->startMin,0,isdst);
 
 		if (rtime < atime) //past date/time
 		{
 			//schedule for next month
 			ltime.tm_mon++;
+			if (ltime.tm_mon == 12) // fix for roll over to next year
+			{
+				ltime.tm_mon = 0;
+				ltime.tm_year++;
+			}
 			Month = static_cast<boost::gregorian::months_of_year>(ltime.tm_mon + 1);
 			nth_dow ndm(Occurence, Day, Month);
 			boost::gregorian::date d = ndm.get_date(ltime.tm_year + 1900);
-			ltime.tm_mday = d.day();
-			rtime = mktime(&ltime) + (roffset * 60);
+
+			constructTime(rtime,tm1,ltime.tm_year+1900,ltime.tm_mon,d.day(),pItem->startHour,pItem->startMin,0,isdst);
 		}
 
+		rtime += roffset * 60; // add randomness
 		pItem->startTime = rtime;
 		return true;
 	}
 	else if (pItem->timerType == TTYPE_YEARLY)
 	{
-		ltime.tm_mday = pItem->MDay;
-		ltime.tm_mon = pItem->Month - 1;
-		ltime.tm_hour = pItem->startHour;
-		ltime.tm_min = pItem->startMin;
-		//if mday exceeds max days in month, find next year with this amount of days
-		while (ltime.tm_mday > boost::gregorian::gregorian_calendar::end_of_month_day(ltime.tm_year + 1900, ltime.tm_mon + 1))
-		{
-			ltime.tm_year++;
-		}
-		rtime = mktime(&ltime) + (roffset * 60);
-		if (rtime < atime) //past date/time
+		constructTime(rtime,tm1,ltime.tm_year+1900,pItem->Month,pItem->MDay,pItem->startHour,pItem->startMin,0,isdst);
+
+		while ( (rtime < atime) || (tm1.tm_mday != pItem->MDay) ) // past date/time OR mday exceeds max days in month
 		{
 			//schedule for next year
-			boost::gregorian::year_iterator m_itr(boost::gregorian::date(ltime.tm_year + 1900, ltime.tm_mon + 1, ltime.tm_mday));
-			++m_itr;
-			ltime.tm_year = m_itr->year() - 1900;
-			rtime = mktime(&ltime) + (roffset * 60);
+			ltime.tm_year++;
+			constructTime(rtime,tm1,ltime.tm_year+1900,pItem->Month,pItem->MDay,pItem->startHour,pItem->startMin,0,isdst);
 		}
 
+		rtime += roffset * 60; // add randomness
 		pItem->startTime = rtime;
 		return true;
 	}
 	else if (pItem->timerType == TTYPE_YEARLY_WD)
 	{
-		ltime.tm_hour = pItem->startHour;
-		ltime.tm_min = pItem->startMin;
-		ltime.tm_mon = pItem->Month - 1;
-
 		//pItem->Days: mon=1 .. sat=32, sun=64
 		//convert to : sun=0, mon=1 .. sat=6
 		int daynum = (int)log2(pItem->Days) + 1;
@@ -577,35 +543,32 @@ bool CScheduler::AdjustScheduleItem(tScheduleItem *pItem, bool bForceAddDay)
 		nth_dow ndm(Occurence, Day, Month);
 		boost::gregorian::date d = ndm.get_date(ltime.tm_year + 1900);
 
-		ltime.tm_mday = d.day();
-		rtime = mktime(&ltime) + (roffset * 60);
+		constructTime(rtime, tm1, ltime.tm_year + 1900, pItem->Month, d.day(), pItem->startHour, pItem->startMin, 0, isdst);
 
 		if (rtime < atime) //past date/time
 		{
 			//schedule for next year
 			ltime.tm_year++;
-			boost::gregorian::date d = ndm.get_date(ltime.tm_year + 1900);
-			ltime.tm_mday = d.day();
-			rtime = mktime(&ltime) + (roffset * 60);
+			constructTime(rtime, tm1, ltime.tm_year + 1900, pItem->Month, d.day(), pItem->startHour, pItem->startMin, 0, isdst);
 		}
 
+		rtime += roffset * 60; // add randomness
 		pItem->startTime = rtime;
 		return true;
 	}
 	else
 		return false; //unknown timer type
 
-	if (bForceAddDay)
-	{
-		//item is scheduled for next day
-		rtime += (24 * 3600);
-	}
-
-	//Adjust timer by 1 day if we are in the past
-	while (rtime < atime + 60)
-	{
-		rtime += (24 * 3600);
-	}
+	// Adjust timer by 1 day if item is scheduled for next day or we are in the past
+        while (bForceAddDay || (rtime < atime + 60) )
+        {
+                if (tm1.tm_isdst == -1) // rtime was loaded from sunset/sunrise values; need to initialize tm1
+                        localtime_r(&rtime, &tm1);
+                struct tm tm2;
+                tm1.tm_mday++;
+                constructTime(rtime, tm2, tm1.tm_year + 1900, tm1.tm_mon + 1, tm1.tm_mday, tm1.tm_hour, tm1.tm_min, tm1.tm_sec, isdst);
+                bForceAddDay = false;
+        }
 
 	pItem->startTime = rtime;
 	return true;
@@ -727,11 +690,11 @@ void CScheduler::CheckSchedules()
 				strftime(ltimeBuf, sizeof(ltimeBuf), "%Y-%m-%d %H:%M:%S", &ltime);
 
 				if (itt->bIsScene == true)
-					_log.Log(LOG_STATUS, "Schedule item started! Name: %s, Type: %s, SceneID: %llu, Time: %s", itt->DeviceName.c_str(), Timer_Type_Desc(itt->timerType), itt->RowID, ltimeBuf);
+					_log.Log(LOG_STATUS, "Schedule item started! Name: %s, Type: %s, SceneID: %" PRIu64 ", Time: %s", itt->DeviceName.c_str(), Timer_Type_Desc(itt->timerType), itt->RowID, ltimeBuf);
 				else if (itt->bIsThermostat == true)
-					_log.Log(LOG_STATUS, "Schedule item started! Name: %s, Type: %s, ThermostatID: %llu, Time: %s", itt->DeviceName.c_str(), Timer_Type_Desc(itt->timerType), itt->RowID, ltimeBuf);
+					_log.Log(LOG_STATUS, "Schedule item started! Name: %s, Type: %s, ThermostatID: %" PRIu64 ", Time: %s", itt->DeviceName.c_str(), Timer_Type_Desc(itt->timerType), itt->RowID, ltimeBuf);
 				else
-					_log.Log(LOG_STATUS, "Schedule item started! Name: %s, Type: %s, DevID: %llu, Time: %s", itt->DeviceName.c_str(), Timer_Type_Desc(itt->timerType), itt->RowID, ltimeBuf);
+					_log.Log(LOG_STATUS, "Schedule item started! Name: %s, Type: %s, DevID: %" PRIu64 ", Time: %s", itt->DeviceName.c_str(), Timer_Type_Desc(itt->timerType), itt->RowID, ltimeBuf);
 				std::string switchcmd = "";
 				if (itt->timerCmd == TCMD_ON)
 					switchcmd = "On";
@@ -758,7 +721,7 @@ void CScheduler::CheckSchedules()
 */
 						if (!m_mainworker.SwitchScene(itt->RowID, switchcmd))
 						{
-							_log.Log(LOG_ERROR, "Error switching Scene command, SceneID: %llu, Time: %s", itt->RowID, ltimeBuf);
+							_log.Log(LOG_ERROR, "Error switching Scene command, SceneID: %" PRIu64 ", Time: %s", itt->RowID, ltimeBuf);
 						}
 					}
 					else if (itt->bIsThermostat == true)
@@ -767,14 +730,14 @@ void CScheduler::CheckSchedules()
 						sstr << itt->RowID;
 						if (!m_mainworker.SetSetPoint(sstr.str(), itt->Temperature))
 						{
-							_log.Log(LOG_ERROR, "Error setting thermostat setpoint, ThermostatID: %llu, Time: %s", itt->RowID, ltimeBuf);
+							_log.Log(LOG_ERROR, "Error setting thermostat setpoint, ThermostatID: %" PRIu64 ", Time: %s", itt->RowID, ltimeBuf);
 						}
 					}
 					else
 					{
 						//Get SwitchType
 						std::vector<std::vector<std::string> > result;
-						result = m_sql.safe_query("SELECT Type,SubType,SwitchType FROM DeviceStatus WHERE (ID == %llu)",
+						result = m_sql.safe_query("SELECT Type,SubType,SwitchType FROM DeviceStatus WHERE (ID == %" PRIu64 ")",
 							itt->RowID);
 						if (result.size() > 0)
 						{
@@ -791,7 +754,20 @@ void CScheduler::CheckSchedules()
 
 							GetLightStatus(dType, dSubType, switchtype, 0, "", lstatus, llevel, bHaveDimmer, maxDimLevel, bHaveGroupCmd);
 							int ilevel = maxDimLevel;
-							if (((switchtype == STYPE_Dimmer) || (switchtype == STYPE_BlindsPercentage) || (switchtype == STYPE_BlindsPercentageInverted)) && (maxDimLevel != 0))
+							if ((switchtype == STYPE_BlindsPercentage) || (switchtype == STYPE_BlindsPercentageInverted))
+							{
+								if (itt->timerCmd == TCMD_ON)
+								{
+									switchcmd = "Set Level";
+									float fLevel = (maxDimLevel / 100.0f)*itt->Level;
+									if (fLevel > 100)
+										fLevel = 100;
+									ilevel = int(fLevel) + 1;
+								}
+								else if (itt->timerCmd == TCMD_OFF)
+									ilevel = 0;
+							}
+							else if ((switchtype == STYPE_Dimmer) && (maxDimLevel != 0))
 							{
 								if (itt->timerCmd == TCMD_ON)
 								{
@@ -811,7 +787,7 @@ void CScheduler::CheckSchedules()
 							}
 							if (!m_mainworker.SwitchLight(itt->RowID, switchcmd, ilevel, itt->Hue,false,0))
 							{
-								_log.Log(LOG_ERROR, "Error sending switch command, DevID: %llu, Time: %s", itt->RowID, ltimeBuf);
+								_log.Log(LOG_ERROR, "Error sending switch command, DevID: %" PRIu64 ", Time: %s", itt->RowID, ltimeBuf);
 							}
 						}
 					}
@@ -861,7 +837,7 @@ void CScheduler::DeleteExpiredTimers()
 			);
 		iExpiredTimers += result.size();
 	}
-	
+
 	// Check SceneTimers
 	result = m_sql.safe_query("SELECT ID FROM SceneTimers WHERE (Type == %i AND ((Date < '%q') OR (Date == '%q' AND Time < '%q')))",
 		TTYPE_FIXEDDATETIME,
@@ -1024,7 +1000,7 @@ namespace http {
 		}
 		void CWebServer::RType_Timers(WebEmSession & session, const request& req, Json::Value &root)
 		{
-			unsigned long long idx = 0;
+			uint64_t idx = 0;
 			if (request::findValue(&req, "idx") != "")
 			{
 				std::stringstream s_str(request::findValue(&req, "idx"));
@@ -1037,7 +1013,7 @@ namespace http {
 			char szTmp[50];
 
 			std::vector<std::vector<std::string> > result;
-			result = m_sql.safe_query("SELECT ID, Active, [Date], Time, Type, Cmd, Level, Hue, Days, UseRandomness, MDay, Month, Occurence FROM Timers WHERE (DeviceRowID==%llu) AND (TimerPlan==%d) ORDER BY ID",
+			result = m_sql.safe_query("SELECT ID, Active, [Date], Time, Type, Cmd, Level, Hue, Days, UseRandomness, MDay, Month, Occurence FROM Timers WHERE (DeviceRowID==%" PRIu64 ") AND (TimerPlan==%d) ORDER BY ID",
 				idx, m_sql.m_ActiveTimerPlan);
 			if (result.size() > 0)
 			{
@@ -1086,16 +1062,21 @@ namespace http {
 		{
 			if (session.rights != 2)
 			{
-				//No admin user, and not allowed to be here
-				return;
+				session.reply_status = reply::forbidden;
+				return; //Only admin user allowed
 			}
 
 			int rnOldvalue = 0;
 			int rnvalue = 0;
 			m_sql.GetPreferencesVar("ActiveTimerPlan", rnOldvalue);
 			rnvalue = atoi(request::findValue(&req, "ActiveTimerPlan").c_str());
-			if ((rnOldvalue != rnvalue) && ((rnvalue==0) || (rnvalue==1)))
+			if (rnOldvalue != rnvalue)
 			{
+				std::vector<std::vector<std::string> > result;
+				result = m_sql.safe_query("SELECT Name FROM TimerPlans WHERE (ID == %d)", rnvalue);
+				if (result.empty())
+					return; //timerplan not found!
+				_log.Log(LOG_STATUS, "Scheduler Timerplan changed (%d - %s)", rnvalue, result[0][0].c_str());
 				m_sql.UpdatePreferencesVar("ActiveTimerPlan", rnvalue);
 				m_sql.m_ActiveTimerPlan = rnvalue;
 				m_mainworker.m_scheduler.ReloadSchedules();
@@ -1109,8 +1090,8 @@ namespace http {
 		{
 			if (session.rights != 2)
 			{
-				//No admin user, and not allowed to be here
-				return;
+				session.reply_status = reply::forbidden;
+				return; //Only admin user allowed
 			}
 
 			std::string idx = request::findValue(&req, "idx");
@@ -1214,8 +1195,8 @@ namespace http {
 		{
 			if (session.rights != 2)
 			{
-				//No admin user, and not allowed to be here
-				return;
+				session.reply_status = reply::forbidden;
+				return; //Only admin user allowed
 			}
 
 			std::string idx = request::findValue(&req, "idx");
@@ -1318,8 +1299,8 @@ namespace http {
 		{
 			if (session.rights != 2)
 			{
-				//No admin user, and not allowed to be here
-				return;
+				session.reply_status = reply::forbidden;
+				return; //Only admin user allowed
 			}
 
 			std::string idx = request::findValue(&req, "idx");
@@ -1338,8 +1319,8 @@ namespace http {
 		{
 			if (session.rights != 2)
 			{
-				//No admin user, and not allowed to be here
-				return;
+				session.reply_status = reply::forbidden;
+				return; //Only admin user allowed
 			}
 
 			std::string idx = request::findValue(&req, "idx");
@@ -1358,8 +1339,8 @@ namespace http {
 		{
 			if (session.rights != 2)
 			{
-				//No admin user, and not allowed to be here
-				return;
+				session.reply_status = reply::forbidden;
+				return; //Only admin user allowed
 			}
 
 			std::string idx = request::findValue(&req, "idx");
@@ -1378,8 +1359,8 @@ namespace http {
 		{
 			if (session.rights != 2)
 			{
-				//No admin user, and not allowed to be here
-				return;
+				session.reply_status = reply::forbidden;
+				return; //Only admin user allowed
 			}
 
 			std::string idx = request::findValue(&req, "idx");
@@ -1396,7 +1377,7 @@ namespace http {
 
 		void CWebServer::RType_SetpointTimers(WebEmSession & session, const request& req, Json::Value &root)
 		{
-			unsigned long long idx = 0;
+			uint64_t idx = 0;
 			if (request::findValue(&req, "idx") != "")
 			{
 				std::stringstream s_str(request::findValue(&req, "idx"));
@@ -1409,7 +1390,7 @@ namespace http {
 			char szTmp[50];
 
 			std::vector<std::vector<std::string> > result;
-			result = m_sql.safe_query("SELECT ID, Active, [Date], Time, Type, Temperature, Days, MDay, Month, Occurence FROM SetpointTimers WHERE (DeviceRowID=%llu) AND (TimerPlan==%d) ORDER BY ID",
+			result = m_sql.safe_query("SELECT ID, Active, [Date], Time, Type, Temperature, Days, MDay, Month, Occurence FROM SetpointTimers WHERE (DeviceRowID=%" PRIu64 ") AND (TimerPlan==%d) ORDER BY ID",
 				idx, m_sql.m_ActiveTimerPlan);
 			if (result.size() > 0)
 			{
@@ -1450,8 +1431,8 @@ namespace http {
 		{
 			if (session.rights != 2)
 			{
-				//No admin user, and not allowed to be here
-				return;
+				session.reply_status = reply::forbidden;
+				return; //Only admin user allowed
 			}
 
 			std::string idx = request::findValue(&req, "idx");
@@ -1546,8 +1527,8 @@ namespace http {
 		{
 			if (session.rights != 2)
 			{
-				//No admin user, and not allowed to be here
-				return;
+				session.reply_status = reply::forbidden;
+				return; //Only admin user allowed
 			}
 
 			std::string idx = request::findValue(&req, "idx");
@@ -1641,8 +1622,8 @@ namespace http {
 		{
 			if (session.rights != 2)
 			{
-				//No admin user, and not allowed to be here
-				return;
+				session.reply_status = reply::forbidden;
+				return; //Only admin user allowed
 			}
 
 			std::string idx = request::findValue(&req, "idx");
@@ -1661,8 +1642,8 @@ namespace http {
 		{
 			if (session.rights != 2)
 			{
-				//No admin user, and not allowed to be here
-				return;
+				session.reply_status = reply::forbidden;
+				return; //Only admin user allowed
 			}
 
 			std::string idx = request::findValue(&req, "idx");
@@ -1681,8 +1662,8 @@ namespace http {
 		{
 			if (session.rights != 2)
 			{
-				//No admin user, and not allowed to be here
-				return;
+				session.reply_status = reply::forbidden;
+				return; //Only admin user allowed
 			}
 
 			std::string idx = request::findValue(&req, "idx");
@@ -1701,8 +1682,8 @@ namespace http {
 		{
 			if (session.rights != 2)
 			{
-				//No admin user, and not allowed to be here
-				return;
+				session.reply_status = reply::forbidden;
+				return; //Only admin user allowed
 			}
 
 			std::string idx = request::findValue(&req, "idx");
@@ -1719,7 +1700,7 @@ namespace http {
 
 		void CWebServer::RType_SceneTimers(WebEmSession & session, const request& req, Json::Value &root)
 		{
-			unsigned long long idx = 0;
+			uint64_t idx = 0;
 			if (request::findValue(&req, "idx") != "")
 			{
 				std::stringstream s_str(request::findValue(&req, "idx"));
@@ -1733,7 +1714,7 @@ namespace http {
 			char szTmp[40];
 
 			std::vector<std::vector<std::string> > result;
-			result = m_sql.safe_query("SELECT ID, Active, [Date], Time, Type, Cmd, Level, Hue, Days, UseRandomness, MDay, Month, Occurence FROM SceneTimers WHERE (SceneRowID==%llu) AND (TimerPlan==%d) ORDER BY ID",
+			result = m_sql.safe_query("SELECT ID, Active, [Date], Time, Type, Cmd, Level, Hue, Days, UseRandomness, MDay, Month, Occurence FROM SceneTimers WHERE (SceneRowID==%" PRIu64 ") AND (TimerPlan==%d) ORDER BY ID",
 				idx, m_sql.m_ActiveTimerPlan);
 			if (result.size() > 0)
 			{
@@ -1782,8 +1763,8 @@ namespace http {
 		{
 			if (session.rights != 2)
 			{
-				//No admin user, and not allowed to be here
-				return;
+				session.reply_status = reply::forbidden;
+				return; //Only admin user allowed
 			}
 
 			std::string idx = request::findValue(&req, "idx");
@@ -1884,8 +1865,8 @@ namespace http {
 		{
 			if (session.rights != 2)
 			{
-				//No admin user, and not allowed to be here
-				return;
+				session.reply_status = reply::forbidden;
+				return; //Only admin user allowed
 			}
 
 			std::string idx = request::findValue(&req, "idx");
@@ -1986,8 +1967,8 @@ namespace http {
 		{
 			if (session.rights != 2)
 			{
-				//No admin user, and not allowed to be here
-				return;
+				session.reply_status = reply::forbidden;
+				return; //Only admin user allowed
 			}
 
 			std::string idx = request::findValue(&req, "idx");
@@ -2006,8 +1987,8 @@ namespace http {
 		{
 			if (session.rights != 2)
 			{
-				//No admin user, and not allowed to be here
-				return;
+				session.reply_status = reply::forbidden;
+				return; //Only admin user allowed
 			}
 
 			std::string idx = request::findValue(&req, "idx");
@@ -2026,8 +2007,8 @@ namespace http {
 		{
 			if (session.rights != 2)
 			{
-				//No admin user, and not allowed to be here
-				return;
+				session.reply_status = reply::forbidden;
+				return; //Only admin user allowed
 			}
 
 			std::string idx = request::findValue(&req, "idx");
@@ -2046,8 +2027,8 @@ namespace http {
 		{
 			if (session.rights != 2)
 			{
-				//No admin user, and not allowed to be here
-				return;
+				session.reply_status = reply::forbidden;
+				return; //Only admin user allowed
 			}
 
 			std::string idx = request::findValue(&req, "idx");
