@@ -76,9 +76,10 @@ const char *szHelp=
 #ifdef WWW_ENABLE_SSL
 	"\t-sslwww port (for example -sslwww 443, or -sslwww 0 to disable https)\n"
 	"\t-sslcert file_path (for example /opt/domoticz/server_cert.pem)\n"
+	"\t-sslkey file_path (if different from certificate file)\n"
 	"\t-sslpass passphrase (to access to server private key in certificate)\n"
 	"\t-sslmethod method (for SSL method)\n"
-	"\t-ssloptions options (for SSL options, default is 'default_workarounds,no_sslv2,single_dh_use')\n"
+	"\t-ssloptions options (for SSL options, default is 'default_workarounds,no_sslv2,no_sslv3,no_tlsv1,no_tlsv1_1,single_dh_use')\n"
 	"\t-ssldhparam file_path (for SSL DH parameters)\n"
 #endif
 #if defined WIN32
@@ -103,7 +104,8 @@ const char *szHelp=
 #else
 	"\t-log file_path (for example /var/log/domoticz.log)\n"
 #endif
-	"\t-loglevel (0=All, 1=Status+Error, 2=Error)\n"
+	"\t-loglevel (0=All, 1=Status+Error, 2=Error , 3= Trace )\n"
+	"\t-debug    allow log trace level 3 \n"
 	"\t-notimestamps (do not prepend timestamps to logs; useful with syslog, etc.)\n"
 	"\t-php_cgi_path (for example /usr/bin/php-cgi)\n"
 #ifndef WIN32
@@ -160,6 +162,7 @@ CLogger _log;
 http::server::CWebServerHelper m_webservers;
 CSQLHelper m_sql;
 CNotificationHelper m_notifications;
+
 std::string logfile = "";
 bool g_bStopApplication = false;
 bool g_bUseSyslog = false;
@@ -226,13 +229,6 @@ void daemonize(const char *rundir, const char *pidfile)
 	struct sigaction newSigAction;
 	sigset_t newSigSet;
 
-	/* Check if parent process id is set */
-	if (getppid() == 1)
-	{
-		/* PPID exists, therefore we are already a daemon */
-		return;
-	}
-
 	/* Set signal mask - signals we want to block */
 	sigemptyset(&newSigSet);
 	sigaddset(&newSigSet, SIGCHLD);  /* ignore child - i.e. we don't need to wait for it */
@@ -253,7 +249,7 @@ void daemonize(const char *rundir, const char *pidfile)
 	sigaction(SIGABRT, &newSigAction, NULL);    // catch abnormal termination signal
 	sigaction(SIGILL,  &newSigAction, NULL);    // catch invalid program image
 #ifndef WIN32
-	sigaction(SIGHUP,  &newSigAction, NULL);    // catch HUP, for logrotation
+	sigaction(SIGHUP,  &newSigAction, NULL);    // catch HUP, for log rotation
 #endif
 	
 	/* Fork*/
@@ -296,7 +292,7 @@ void daemonize(const char *rundir, const char *pidfile)
 	int twrite=write(pidFilehandle, str, strlen(str));
 	if (twrite != strlen(str))
 	{
-		syslog(LOG_INFO, "Could not write to lockfile %s, exiting", pidfile);
+		syslog(LOG_INFO, "Could not write to lock file %s, exiting", pidfile);
 		exit(EXIT_FAILURE);
 	}
 
@@ -376,6 +372,87 @@ void daemonize(const char *rundir, const char *pidfile)
 		sysctl(mib, 4, pathName, &cb, NULL, 0);
 		return cb;
 	}
+#elif defined(__OpenBSD__)
+#include <sys/sysctl.h>
+static size_t getExecutablePathName(char* pathName, size_t pathNameCapacity)
+{
+        int mib[4];
+        char **argv;
+        size_t len = 0;
+        const char *comm;
+
+        mib[0] = CTL_KERN;
+        mib[1] = KERN_PROC_ARGS;
+        mib[2] = getpid();
+        mib[3] = KERN_PROC_ARGV;
+        pathName[0] = '\0';
+        if (sysctl(mib, 4, NULL, &len, NULL, 0) < 0)
+        {
+                return 0;
+        }
+
+        if (!(argv = (char**)malloc(len)))
+        {
+                return 0;
+        }
+
+        if (sysctl(mib, 4, argv, &len, NULL, 0) < 0)
+        {
+                len = 0;
+                goto finally;
+        }
+
+	        len = 0;
+        comm = argv[0];
+
+        if (*comm == '/' || *comm == '.')
+        {
+                // In OpenBSD PATH_MAX is 1024
+                char * fullPath = (char*) malloc(PATH_MAX);
+                if(!fullPath)
+                {
+                        goto finally;
+                }
+
+                if (realpath(comm, fullPath))
+                {
+                        if(pathNameCapacity > strnlen(fullPath, PATH_MAX))
+                        {
+                                strlcpy(pathName, fullPath, pathNameCapacity);
+                                len = strnlen(pathName, pathNameCapacity);
+                                free(fullPath);
+                        }
+                }
+        }
+        else
+        {
+                char *sp;
+                char *xpath = strdup(getenv("PATH"));
+                char *path = strtok_r(xpath, ":", &sp);
+                struct stat st;
+
+		if (!xpath)
+                {
+                        goto finally;
+                }
+
+                while (path)
+                {
+                        snprintf(pathName, pathNameCapacity, "%s/%s", path, comm);
+                        if (!stat(pathName, &st) && (st.st_mode & S_IXUSR))
+                        {
+                                break;
+                        }
+                        pathName[0] = '\0';
+                        path = strtok_r(NULL, ":", &sp);
+                }
+                free(xpath);
+        }
+
+  finally:
+        free(argv);
+        return len;
+}
 #elif defined(__APPLE__) /* elif of: #elif defined(__linux__) */
 	#include <mach-o/dyld.h>
 	static size_t getExecutablePathName(char* pathName, size_t pathNameCapacity)
@@ -461,8 +538,6 @@ int main(int argc, char**argv)
 			_log.Log(LOG_ERROR, "Please specify an output log file");
 			return 1;
 		}
-		logfile = cmdLine.GetSafeArgument("-log", 0, "domoticz.log");
-		_log.SetOutputFile(logfile.c_str());
 	}
 	if (cmdLine.HasSwitch("-loglevel"))
 	{
@@ -471,9 +546,19 @@ int main(int argc, char**argv)
 			_log.Log(LOG_ERROR, "Please specify logfile output level (0=All, 1=Status+Error, 2=Error)");
 			return 1;
 		}
-		int Level = atoi(cmdLine.GetSafeArgument("-loglevel", 0, "").c_str());
-		_log.SetVerboseLevel((_eLogFileVerboseLevel)Level);
 	}
+	if (cmdLine.HasSwitch("-verbose"))
+	{
+		if (cmdLine.GetArgumentCount("-verbose") != 1)
+		{
+			_log.Log(LOG_ERROR, "Please specify a verbose level");
+			return 1;
+		}
+	}
+	if (cmdLine.HasSwitch("-debug"))
+		_log.SetLogDebug(true);
+	else
+		_log.SetLogDebug(false);
 	if (cmdLine.HasSwitch("-notimestamps"))
 	{
 		_log.EnableLogTimestamps(false);
@@ -564,7 +649,7 @@ int main(int argc, char**argv)
 		else if (file_exist("/sys/devices/virtual/thermal/thermal_zone0/temp"))
 		{
 			//_log.Log(LOG_STATUS,"System: ODroid");
-			szInternalTemperatureCommand="cat /sys/devices/virtual/thermal/thermal_zone0/temp | awk '{ printf (\"temp=%0.2f\\n\",$1/1000); }'";
+			szInternalTemperatureCommand="cat /sys/devices/virtual/thermal/thermal_zone0/temp | awk '{ if ($1 < 100) printf(\"temp=%d\\n\",$1); else printf (\"temp=%0.2f\\n\",$1/1000); }'";
 			bHasInternalTemperature = true;
 		}
 	}
@@ -578,6 +663,15 @@ int main(int argc, char**argv)
 		szInternalCurrentCommand = "cat /sys/class/power_supply/ac/current_now | awk '{ printf (\"curr=%0.2f\\n\",$1/1000000); }'";
 		bHasInternalCurrent = true;
 	}
+#if defined (__OpenBSD__)
+
+	szInternalTemperatureCommand="sysctl hw.sensors.acpitz0.temp0|sed -e 's/.*temp0/temp/'|cut -d ' ' -f 1";
+	bHasInternalTemperature = true;
+	szInternalVoltageCommand = "sysctl hw.sensors.acpibat0.volt1|sed -e 's/.*volt1/volt/'|cut -d ' ' -f 1";
+	bHasInternalVoltage = true;
+	//bHasInternalCurrent = true;
+
+#endif
 	_log.Log(LOG_STATUS,"Startup Path: %s", szStartupFolder.c_str());
 #endif
 
@@ -633,6 +727,12 @@ int main(int argc, char**argv)
 			return 1;
 		}
 		std::string wwwport = cmdLine.GetSafeArgument("-www", 0, "");
+		int iPort = (int)atoi(wwwport.c_str());
+		if ((iPort < 0) || (iPort > 32767))
+		{
+			_log.Log(LOG_ERROR, "Please specify a valid www port");
+			return 1;
+		}
 		webserver_settings.listening_port = wwwport;
 	}
 
@@ -668,6 +768,12 @@ int main(int argc, char**argv)
 			return 1;
 		}
 		std::string wwwport = cmdLine.GetSafeArgument("-sslwww", 0, "");
+		int iPort = (int)atoi(wwwport.c_str());
+		if ((iPort < 0) || (iPort > 32767))
+		{
+			_log.Log(LOG_ERROR, "Please specify a valid sslwww port");
+			return 1;
+		}
 		secure_webserver_settings.listening_port = wwwport;
 	}
 	if (!webserver_settings.listening_address.empty()) {
@@ -682,6 +788,16 @@ int main(int argc, char**argv)
 			return 1;
 		}
 		secure_webserver_settings.cert_file_path = cmdLine.GetSafeArgument("-sslcert", 0, "");
+		secure_webserver_settings.private_key_file_path = secure_webserver_settings.cert_file_path;
+	}
+	if (cmdLine.HasSwitch("-sslkey"))
+	{
+		if (cmdLine.GetArgumentCount("-sslkey") != 1)
+		{
+			_log.Log(LOG_ERROR, "Please specify a file path for your server SSL key file");
+			return 1;
+		}
+		secure_webserver_settings.private_key_file_path = cmdLine.GetSafeArgument("-sslkey", 0, "");
 	}
 	if (cmdLine.HasSwitch("-sslpass"))
 	{
@@ -790,16 +906,6 @@ int main(int argc, char**argv)
 			szWebRoot = szroot;
 	}
 
-	if (cmdLine.HasSwitch("-verbose"))
-	{
-		if (cmdLine.GetArgumentCount("-verbose") != 1)
-		{
-			_log.Log(LOG_ERROR, "Please specify a verbose level");
-			return 1;
-		}
-		int Level = atoi(cmdLine.GetSafeArgument("-verbose", 0, "").c_str());
-		m_mainworker.SetVerboseLevel((eVerboseLevel)Level);
-	}
 #if defined WIN32
 	if (cmdLine.HasSwitch("-nobrowser"))
 	{
@@ -899,6 +1005,28 @@ int main(int argc, char**argv)
 	}
 	m_StartTime = time(NULL);
 
+  //set log level / log output file name verbose level if set on command line
+  //the value as been taken from database in call of GetLogPreference m_mainworker.Start()
+	if (cmdLine.HasSwitch("-log"))
+	{
+		logfile = cmdLine.GetSafeArgument("-log", 0, "domoticz.log");
+		_log.SetOutputFile(logfile.c_str());
+	}
+	if (cmdLine.HasSwitch("-loglevel"))
+	{
+		int Level = atoi(cmdLine.GetSafeArgument("-loglevel", 0, "").c_str());
+		if     (Level==0) _log.SetVerboseLevel(VBL_ALL);
+		else if(Level==1) _log.SetVerboseLevel(VBL_STATUS_ERROR);
+		else if(Level==2) _log.SetVerboseLevel(VBL_ERROR);
+		else if ((Level == 3) && (_log.GetLogDebug())) _log.SetVerboseLevel(VBL_TRACE);
+	}
+	if (cmdLine.HasSwitch("-verbose"))
+	{
+		int Level = atoi(cmdLine.GetSafeArgument("-verbose", 0, "").c_str());
+		m_mainworker.SetVerboseLevel((eVerboseLevel)Level);
+	}
+
+
 	/* now, lets get into an infinite loop of doing nothing. */
 #if defined WIN32
 #ifndef _DEBUG
