@@ -1,11 +1,11 @@
 define(['app'], function (app) {
-	app.controller('RestoreDatabaseController', [ '$scope', '$rootScope', '$window', '$location', '$http', '$interval', function($scope,$rootScope,$window,$location,$http,$interval) {
-	
-		$scope.selected_file="";
-	
+	app.controller('RestoreDatabaseController', ['$scope', '$rootScope', '$window', '$location', '$http', '$interval', function ($scope, $rootScope, $window, $location, $http, $interval) {
+
+		$scope.selected_file = "";
+
 		init();
 
-		$scope.uploadFile = function() {
+		$scope.uploadFile = function () {
 			$http({
 				method: 'POST',
 				url: 'restoredatabase.webem',
@@ -28,24 +28,23 @@ define(['app'], function (app) {
 					return formData;
 				}
 			})
-			.success(function (data) {
-				$window.location = '/#Dashboard';
-			})
-			.error(function (data, status) {
-				$window.location = '/#Dashboard';
-			});
-   		};
-		
-		$scope.onSubmit = function() {
+				.success(function (data) {
+					$window.location = '/#Dashboard';
+				})
+				.error(function (data, status) {
+					$window.location = '/#Dashboard';
+				});
+		};
+
+		$scope.onSubmit = function () {
 			if (typeof $scope.file == 'undefined') {
 				return;
 			}
 			$scope.uploadFile();
 		}
 
-		function init()
-		{
+		function init() {
 			$('#maincontent').i18n();
 		};
-	} ]);
+	}]);
 });
\ No newline at end of file
